import os
path = os.path
from random import randrange

from myhdl import (block, Signal, intbv, delay, always_comb,
                   instance, StopSimulation, conversion
                   )
D = 256

ROM = tuple([randrange(D) for __ in range(D)])


@block
def rom1(dout, addr, clk):

    @instance
    def rdLogic():
        while 1:
            yield clk.posedge
            dout.next = ROM[int(addr)]

    return rdLogic


@block
def rom2(dout, addr, clk):

    theROM = ROM

    @instance
    def rdLogic():
        while 1:
            yield clk.posedge
            dout.next = theROM[int(addr)]

    return rdLogic


@block
def rom3(dout, addr, clk):

    @instance
    def rdLogic():
        tmp = intbv(0)[8:]
        while 1:
            yield addr
            tmp[:] = ROM[int(addr)]
            dout.next = tmp

    return rdLogic


@block
def rom4(dout, addr, clk):

    @always_comb
    def read():
        dout.next = ROM[int(addr)]

    return read


@block
def RomBench(rom):

    dout = Signal(intbv(0)[8:])
    addr = Signal(intbv(1)[8:])
    clk = Signal(bool(0))

    rom_inst = rom(dout, addr, clk)

    @instance
    def stimulus():
        for i in range(D):
            addr.next = i
            yield clk.negedge
            yield clk.posedge
            yield delay(1)
            if __debug__:
                assert dout == ROM[i]
            print(dout)
        raise StopSimulation()

    @instance
    def clkgen():
        clk.next = 1
        while 1:
            yield delay(10)
            clk.next = not clk

    return clkgen, stimulus, rom_inst


def test1():
    assert conversion.verify(RomBench(rom1)) == 0


def test2():
    assert conversion.verify(RomBench(rom2)) == 0


def test3():
    assert conversion.verify(RomBench(rom3)) == 0


def test4():
    assert conversion.verify(RomBench(rom4)) == 0




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for inferWaiter """
import random
from random import randrange
from types import GeneratorType

from myhdl import Signal, intbv, instance, delay, StopSimulation

from myhdl._Simulation import Simulation
from myhdl._Waiter import (_DelayWaiter, _EdgeTupleWaiter, _EdgeWaiter,
                           _inferWaiter, _SignalTupleWaiter, _SignalWaiter,
                           _Waiter)

random.seed(1)  # random, but deterministic

QUIET = 1


def SignalFunc1(a, b, c, d, r):

    @instance
    def comb():
        while 1:
            yield a
            r.next = a + b + c

    return comb


def SignalFunc2(a, b, c, d, r):

    def comb(a, r):
        while 1:
            yield a
            r.next = a - b + c

    return comb(a, r)


def SignalTupleFunc1(a, b, c, d, r):

    @instance
    def comb():
        while 1:
            yield a, b, c
            r.next = a + b + c

    return comb


def SignalTupleFunc2(a, b, c, d, r):

    def comb(a, r):
        while 1:
            yield a, b, c
            r.next = a - b + c

    return comb(a, r)


def DelayFunc(a, b, c, d, r):

    @instance
    def comb():
        while 1:
            yield delay(3)
            r.next = a + b + c

    return comb


def EdgeFunc1(a, b, c, d, r):

    @instance
    def comb():
        while 1:
            yield c.posedge
            r.next = a + b + c

    return comb


def EdgeFunc2(a, b, c, d, r):

    def comb(c, r):
        while 1:
            yield c.negedge
            r.next = a + b + c
            if a > 5:
                yield c.posedge
                r.next = a - b - c
            else:
                r.next = a + b - c

    return comb(c, r)


def EdgeTupleFunc1(a, b, c, d, r):

    @instance
    def comb():
        while 1:
            yield c.posedge, d.negedge
            r.next = a + b + c

    return comb


def EdgeTupleFunc2(a, b, c, d, r):

    def comb(c, r):
        while 1:
            yield c.negedge, d.posedge
            r.next = a + b + c
            if a > 5:
                yield c.posedge, d.negedge
                r.next = a - b - c
            else:
                r.next = a + b - c

    return comb(c, r)


def GeneralFunc(a, b, c, d, r):

    def comb(c, r):
        while 1:
            yield c.negedge, d.posedge
            r.next = a + b + c
            if a > 5:
                yield c, d.negedge
                r.next = a - b - c
            else:
                r.next = a + b - c

    return comb(c, r)


class TestInferWaiter:

    def bench(self, genFunc, waiterType):

        a, b, c, d, r, s = [Signal(intbv(0)) for __ in range(6)]

        gen_inst_r = genFunc(a, b, c, d, r)
        if not isinstance(gen_inst_r, GeneratorType): # decorator type
            gen_inst_r = gen_inst_r.gen
        assert type(_inferWaiter(gen_inst_r)) == waiterType

        gen_inst_s = genFunc(a, b, c, d, s)
        if not isinstance(gen_inst_s, GeneratorType): # decorator type
            gen_inst_s = gen_inst_s.gen

        def stimulus():
            for dummy in range(1000):
                yield delay(randrange(1, 10))
                if randrange(2):
                    a.next = randrange(32)
                if randrange(2):
                    b.next = randrange(32)
                c.next = randrange(2)
                d.next = randrange(2)
            raise StopSimulation

        def check():
            while 1:
                yield a, b, c, r, s
                assert r == s

        return gen_inst_r, _Waiter(gen_inst_s), _Waiter(stimulus()), _Waiter(check())

    def testSignal1(self):
        sim = Simulation(self.bench(SignalFunc1, _SignalWaiter))
        sim.run()

    def testSignal2(self):
        sim = Simulation(self.bench(SignalFunc2, _SignalWaiter))
        sim.run()

    def testSignalTuple1(self):
        sim = Simulation(self.bench(SignalTupleFunc1, _SignalTupleWaiter))
        sim.run()

    def testSignalTuple2(self):
        sim = Simulation(self.bench(SignalTupleFunc2, _SignalTupleWaiter))
        sim.run()

    def testDelay(self):
        sim = Simulation(self.bench(DelayFunc, _DelayWaiter))
        sim.run()

    def testEdge1(self):
        sim = Simulation(self.bench(EdgeFunc1, _EdgeWaiter))
        sim.run()

    def testEdge2(self):
        sim = Simulation(self.bench(EdgeFunc2, _EdgeWaiter))
        sim.run()

    def testEdgeTuple1(self):
        sim = Simulation(self.bench(EdgeTupleFunc1, _EdgeTupleWaiter))
        sim.run()

    def testEdgeTuple2(self):
        sim = Simulation(self.bench(EdgeTupleFunc2, _EdgeTupleWaiter))
        sim.run()

    def testGeneral(self):
        sim = Simulation(self.bench(GeneralFunc, _Waiter))
        sim.run()



from myhdl import (block, Signal, always)


@block
def bug_3529686(clr, clk, run, ack, serialout):

    @always(clk.posedge, clr.posedge)
    def fsm():
        if (clr == 0):
            serialout.next = 0
        else:
            if (ack == 0):
                serialout.next = 0
            elif (run == 1):
                serialout.next = 1

    return fsm


clr, clk, run, ack, serialout = [Signal(bool()) for i in range(5)]


def test_bug_3529686():
    try:
        bug_3529686(clr, clk, run, ack, serialout).convert(hdl='VHDL')
    except:
        assert False



from myhdl import (block, Signal, intbv, downrange, always_seq, concat, ResetSignal)


@block
def issue_13(reset, clk, d, en, q):
    COSET = 0x55

    def calculateHec(header):
        """ Return hec for an ATM header, represented as an intbv.

        The hec polynomial is 1 + x + x**2 + x**8.
        """
        hec = intbv(0)[8:]

        for ii in downrange(len(header)):
            bit = header[ii]
            hec[8:] = concat(hec[7:2],
                             bit ^ hec[1] ^ hec[7],
                             bit ^ hec[0] ^ hec[7],
                             bit ^ hec[7]
                            )
        return hec ^ COSET

    @always_seq(clk.posedge, reset=reset)
    def comb():
        if en:
            q.next = calculateHec(d)

    return comb


def test_issue_13():

    reset = ResetSignal(0, active=1, isasync=False)
    clk = Signal(bool(0))
    d = Signal(intbv(0)[32:])
    en = Signal(bool(0))
    q = Signal(intbv(0)[8:])

    # toVHDL.numeric_ports = False

    issue_13(reset, clk, d, en, q).analyze_convert() == 0




import myhdl
from myhdl import *

from dff import dff

ACTIVE_LOW, INACTIVE_HIGH = 0, 1

def dff_clkout(clkout, q, d, clk, reset):
    
    DFF_1 = dff(q, d, clkout, reset)

    @instance
    def assign():
        while 1:
            yield clk
            clkout.next = clk
            
    return DFF_1, assign




from myhdl import (block, Signal, intbv, instance)


@block
def bug_3577799 (
        clk,
        reset_clk,
        wr_data,
        wr,
        rd_data
    ):

    @instance
    def seq():
        count = 0
        while True:
            yield clk.posedge
            if reset_clk:
                count = 0
            else:
                if wr:
                    if count < depth:
                        rd_data.next = wr_data
                        count = count + 1

    return seq


depth = 8
clk = Signal(bool(0))
reset_clk = Signal(bool(0))
wr_data = Signal(intbv(0)[16:])
wr = Signal(bool(0))
rd_data = Signal(intbv(0)[16:])


def test_bug_3577799():
    assert bug_3577799(clk, reset_clk, wr_data, wr, rd_data).analyze_convert() == 0



import os
path = os.path

from myhdl import (block, Signal, enum, intbv, delay, always, instance, StopSimulation)

# SEARCH, CONFIRM, SYNC = range(3)
ACTIVE_LOW = bool(0)
FRAME_SIZE = 8
t_State_b = enum('SEARCH', 'CONFIRM', 'SYNC')
t_State_oh = enum('SEARCH', 'CONFIRM', 'SYNC', encoding="one_hot")
t_State_oc = enum('SEARCH', 'CONFIRM', 'SYNC', encoding="one_cold")


@block
def FramerCtrl(SOF, state, syncFlag, clk, reset_n, t_State):

    """ Framing control FSM.

    SOF -- start-of-frame output bit
    state -- FramerState output
    syncFlag -- sync pattern found indication input
    clk -- clock input
    reset_n -- active low reset

    """

    index = Signal(intbv(0)[8:])  # position in frame

    @always(clk.posedge, reset_n.negedge)
    def FSM():
        if reset_n == ACTIVE_LOW:
            SOF.next = 0
            index.next = 0
            state.next = t_State.SEARCH
        else:
            index.next = (index + 1) % FRAME_SIZE
            SOF.next = 0
            if state == t_State.SEARCH:
                index.next = 1
                if syncFlag:
                    state.next = t_State.CONFIRM
            elif state == t_State.CONFIRM:
                if index == 0:
                    if syncFlag:
                        state.next = t_State.SYNC
                    else:
                        state.next = t_State.SEARCH
            elif state == t_State.SYNC:
                if index == 0:
                    if not syncFlag:
                        state.next = t_State.SEARCH
                SOF.next = (index == FRAME_SIZE - 1)
            else:
                raise ValueError("Undefined state")

    return FSM


@block
def FSMBench(FramerCtrl, t_State):

    SOF = Signal(bool(0))
    syncFlag = Signal(bool(0))
    clk = Signal(bool(0))
    reset_n = Signal(bool(1))
    state = Signal(t_State.SEARCH)

    framerctrl_inst = FramerCtrl(SOF, state, syncFlag, clk, reset_n, t_State)

    @instance
    def clkgen():
        clk.next = 0
        reset_n.next = 1
        yield delay(10)
        reset_n.next = 0
        yield delay(10)
        reset_n.next = 1
        yield delay(10)
        for dummy in range(1000):
            yield delay(10)
            clk.next = not clk

    table = (12, 8, 8, 4, 11, 8, 8, 7, 6, 8, 8)

    @instance
    def stimulus():
        for i in range(3):
            yield clk.posedge
        for i in range(len(table)):
            n = table[i]
            syncFlag.next = 1
            yield clk.posedge
            syncFlag.next = 0
            for dummy in range(n - 1):
                yield clk.posedge
        raise StopSimulation

    @instance
    def check():
        yield clk.posedge
        while True:
            yield clk.negedge
            print("negedge")
            # in the end, this should work
            # print state

    return framerctrl_inst, clkgen, stimulus, check


def test():
    assert FSMBench(FramerCtrl, t_State_b).verify_convert() == 0



from myhdl import always_comb


def bin2gray(B, G, width):
    """ Gray encoder.

    B -- input intbv signal, binary encoded
    G -- output intbv signal, gray encoded
    width -- bit width
    """

    @always_comb
    def comb():
        for i in range(width):
            G.next[i] = B[i + 1] ^ B[i]

    return comb



import seven_segment

import myhdl
from myhdl import *

code = [None] * 10
for key, val in seven_segment.encoding.items():
    if 0 <= key <= 9:
        code[key] = int(val, 2)
code = tuple(code)


def bcd2led(led, bcd, clock):

    """ bcd to seven segment led convertor.

    led: seven segment led output
    bcd: bcd input
    clock: clock input

    """

    @always(clock.posedge)
    def synch():
        led.next = code[int(bcd)]

    return synch


def convert():
    led = Signal(intbv(0)[7:])
    bcd = Signal(intbv(0)[4:])
    clock = Signal(False)
    toVerilog(bcd2led, led, bcd, clock)
    toVHDL(bcd2led, led, bcd, clock)


convert()



import os
path = os.path

from myhdl import (block, Signal, intbv, delay, always_comb,
                   always, instance, StopSimulation,
                   conversion
                   )


@block
def ram1(dout, din, addr, we, clk, depth=128):
    """ Simple ram model """

    @instance
    def comb():
        mem = [intbv(0)[8:] for dummy in range(depth)]
        while 1:
            yield clk.posedge
            if we:
                mem[int(addr)][:] = din
            dout.next = mem[int(addr)]

    return comb


@block
def ram_clocked(dout, din, addr, we, clk, depth=128):
    """ Ram model """

    mem = [Signal(intbv(0)[8:]) for __ in range(depth)]

    @instance
    def access():
        while 1:
            yield clk.posedge
            if we:
                mem[int(addr)].next = din
            dout.next = mem[int(addr)]

    return access


@block
def ram_deco1(dout, din, addr, we, clk, depth=128):
    """  Ram model """

    mem = [Signal(intbv(0)[8:]) for __ in range(depth)]

    @instance
    def write():
        while True:
            yield clk.posedge
            if we:
                mem[int(addr)].next = din

    @always_comb
    def read():
        dout.next = mem[int(addr)]

    return write, read


@block
def ram_deco2(dout, din, addr, we, clk, depth=128):
    """  Ram model """

    mem = [Signal(intbv(0)[8:]) for __ in range(depth)]

    @always(clk.posedge)
    def write():
        if we:
            mem[int(addr)].next = din

    @always_comb
    def read():
        dout.next = mem[int(addr)]

    return write, read


@block
def ram2(dout, din, addr, we, clk, depth=128):

    memL = [Signal(intbv()[len(dout):]) for __ in range(depth)]

    @instance
    def wrLogic():
        while 1:
            yield clk.posedge
            if we:
                memL[int(addr)].next = din

    @instance
    def rdLogic():
        while 1:
            yield clk.posedge
            dout.next = memL[int(addr)]

    return wrLogic, rdLogic


@block
def RamBench(ram, depth=128):

    dout = Signal(intbv(0)[8:])
    din = Signal(intbv(0)[8:])
    addr = Signal(intbv(0)[7:])
    we = Signal(bool(0))
    clk = Signal(bool(0))

    mem_inst = ram(dout, din, addr, we, clk, depth)

    @instance
    def stimulus():
        for i in range(depth):
            yield clk.negedge
            din.next = i
            addr.next = i
            we.next = True
            yield clk.negedge
        we.next = False
        for i in range(depth):
            addr.next = i
            yield clk.posedge
            yield delay(1)
            assert dout == i
            print(dout)
        raise StopSimulation()

    @instance
    def clkgen():
        clk.next = 1
        while True:
            yield delay(10)
            clk.next = not clk

    return clkgen, stimulus, mem_inst


def testram_deco1():
    assert conversion.verify(RamBench(ram_deco1)) == 0


def testram_deco2():
    assert conversion.verify(RamBench(ram_deco2)) == 0


def testram_clocked():
    assert conversion.verify(RamBench(ram_clocked)) == 0


def test2():
    assert conversion.verify(RamBench(ram2)) == 0


def test1():
    assert conversion.verify(RamBench(ram1)) == 0



import pytest
import unittest

from myhdl import (always_comb, TristateSignal, Signal, block, instance, delay,
                   instances, StopSimulation, Simulation)
from .util import setupCosimulation


@block
def tristate_obuf(A, Y, OE):
    '''three-state output buffer'''

    Y_d = Y.driver()

    @always_comb
    def hdl():
        Y_d.next = A if OE else None

    return hdl


class OBuf(object):

    def __init__(self):
        self.Y = TristateSignal(True)
        self.A = Signal(False)
        self.OE = Signal(False)

    def interface(self):
        return self.A, self.Y, self.OE


@block
def tristate_obuf_i(obuf):
    '''three-state output buffer, using interface'''

    # Caveat: A local name of the interface signals must be declared,
    #         Otherwise, _HierExtr.extract() will not add them to symdict
    #         and conversion will fail.
    IA, IY, IOE = obuf.interface()
    Y_d = IY.driver()
#     Y_d = obuf.Y.driver()

    @always_comb
    def hdl():
        Y_d.next = IA if IOE else None
#         Y_d.next = obuf.A if obuf.OE else None

    return hdl


class TestTristate(unittest.TestCase):

    def bench(self, obuf=None):
        if obuf:
            tristate_obuf_i(obuf).convert(hdl='Verilog')
            A, Y, OE = obuf.interface()
            inst = setupCosimulation(name='tristate_obuf_i', A=A, Y=Y, OE=OE)
        else:
            Y = TristateSignal(True)
            A = Signal(True)
            OE = Signal(False)
            tristate_obuf(A, Y, OE).convert(hdl='Verilog')
            inst = setupCosimulation(name='tristate_obuf', A=A, Y=Y, OE=OE)

        @instance
        def stimulus():
            yield delay(1)
            # print now(), A, OE, Y
            self.assertEqual(Y, None)

            OE.next = True
            yield delay(1)
            # print now(), A, OE, Y
            self.assertEqual(Y, A)

            A.next = not A
            yield delay(1)
            # print now(), A, OE, Y
            self.assertEqual(Y, A)

            OE.next = False
            yield delay(1)
            # print now(), A, OE, Y
            self.assertEqual(Y, None)

            raise StopSimulation

        return instances()

    @pytest.mark.xfail
    def testOBuf(self):
        sim = Simulation(self.bench())
        sim.run()

    @pytest.mark.xfail
    def testOBufInterface(self):
        obuf = OBuf()
        sim = Simulation(self.bench(obuf))
        sim.run()


if __name__ == '__main__':
    unittest.main()



import myhdl
from myhdl import *

from random_generator import random_generator

def test_randgen():

    random_word = Signal(intbv(0)[31:])
    enable = Signal(bool())
    clock = Signal(bool())
    reset = Signal(bool())
    
    dut = random_generator(random_word, enable, clock, reset)

    @instance
    def stimulus():
        enable.next = 0
        clock.next = 0
        reset.next = 0
        yield delay(10)
        reset.next = 1
        yield delay(10)
        reset.next = 0
        enable.next = 1
        for i in range(2**20):
            yield delay(10)
            clock.next = 1
            yield delay(10)
            clock.next = 0
            print random_word
        
    return dut, stimulus

if __name__ == '__main__':
    tb = Simulation(test_randgen())
    tb.run()




from ._verify import verify, analyze, registerSimulator
from ._toVerilog import toVerilog
from ._toVHDL import toVHDL

__all__ = ["verify",
           "analyze",
           "registerSimulator",
           "toVerilog",
           "toVHDL"
           ]




from myhdl import (block, Signal, ResetSignal, intbv, always_comb, always_seq,
                   instance, delay, StopSimulation, instances)


class Intf1:

    def __init__(self, x):
        self.x = Signal(intbv(0, min=x.min, max=x.max))


class Intf2:

    def __init__(self, y):
        self.y = Signal(intbv(0, min=y.min, max=y.max))


class ZBus:

    def __init__(self, z):
        self.z = Signal(intbv(0, min=z.min, max=z.max))


class Intf3:

    def __init__(self, z):
        self.z = ZBus(z)


class IntfWithConstant1:

    def __init__(self):
        self.const1 = 707
        self.const2 = 3


class IntfWithConstant2:

    def __init__(self):
        self.a = 9
        self.b = 10
        self.c = 1729
        self.more_constants = IntfWithConstant1()


@block
def assign(y, x):

    @always_comb
    def beh_assign():
        y.next = x

    return beh_assign


@block
def assign_intf(x, y):

    @always_comb
    def rtl():
        x.x.next = y.y

    return rtl


@block
def top_assign(x, y, z):
    """
    This module does not test top-level interfaces,
    it only tests intermediate interfaces.
    """
    i1, i2 = Intf1(x), Intf2(y)

    inst1 = assign(x, i1.x)  # x = i1.x
    inst2 = assign(i2.y, y)  # i2.y = y
    inst3 = assign_intf(i1, i2)

    return instances()


@block
def c_testbench_one():
    x, y, z = [Signal(intbv(0, min=-8, max=8))
               for _ in range(3)]

    tb_dut = top_assign(x, y, z)

    @instance
    def tb_stim():
        y.next = 3
        yield delay(10)
        print("x: %d" % (x,))
        assert x == 3

    return tb_dut, tb_stim


@block
def multi_comb(x, y, z):

    @always_comb
    def rtl():
        x.x.next = y.y + z.z.z

    return rtl


@block
def top_multi_comb(x, y, z):
    """
    This module does not test top-level interfaces,
    it only tests intermediate interfaces.
    """
    intf = Intf1(x), Intf2(y), Intf3(z)
    x.assign(intf[0].x)
    intf[1].y.assign(y)
    intf[2].z.z.assign(z)
    inst = multi_comb(*intf)
    return inst


@block
def c_testbench_two():
    x, y, z = [Signal(intbv(0, min=-8, max=8))
               for _ in range(3)]
    tb_dut = top_multi_comb(x, y, z)

    @instance
    def tb_stim():
        y.next = 3
        z.next = 2
        yield delay(10)
        print("x: %d" % (x,))
        assert x == 5

    return tb_dut, tb_stim


@block
def top_const(clock, reset, x, y, intf):

    @always_seq(clock.posedge, reset=reset)
    def rtl1():
        v = intf.a ** 3 + intf.b ** 3
        x.next = v - intf.c

    @always_comb
    def rtl2():
        y.next = x + intf.more_constants.const1 - \
                 intf.more_constants.const2 * 235 - 2

    return rtl1, rtl2


@block
def c_testbench_three():
    """
    this will test the use of constants in an inteface
    as well as top-level interface conversion.
    """
    clock = Signal(bool(0))
    reset = ResetSignal(0, active=0, isasync=True)
    x = Signal(intbv(3, min=-5000, max=5000))
    y = Signal(intbv(4, min=-200, max=200))
    intf = IntfWithConstant2()

    tbdut = top_const(clock, reset, x, y, intf)

    @instance
    def tbclk():
        clock.next = False
        while True:
            yield delay(3)
            clock.next = not clock

    @instance
    def tbstim():
        reset.next = reset.active
        yield delay(33)
        reset.next = not reset.active
        yield clock.posedge
        yield clock.posedge
        print("x: %d" % (x,))
        print("y: %d" % (y,))
        assert x == 0
        assert y == 0
        raise StopSimulation

    return tbdut, tbclk, tbstim


def test_one_analyze():
    x, y, z = [Signal(intbv(0, min=-8, max=8))
               for _ in range(3)]
    # fool name check in convertor
    # to be reviewed
    x._name = 'x'
    y._name = 'y'
    z._name = 'z'
    inst = top_assign(x, y, z)
    assert inst.analyze_convert() == 0


def test_one_verify():
    inst = c_testbench_one()
    assert inst.verify_convert() == 0


def test_two_analyze():
    x, y, z = [Signal(intbv(0, min=-8, max=8))
               for _ in range(3)]
    # fool name check in converter
    # to be reviewed
    x._name = 'x'
    y._name = 'y'
    z._name = 'z'
    inst = top_multi_comb(x, y, z)
    assert inst.analyze_convert() == 0


def test_two_verify():
    inst = c_testbench_two()
    assert inst.verify_convert() == 0


def test_three_analyze():
    clock = Signal(bool(0))
    reset = ResetSignal(0, active=0, isasync=True)
    x = Signal(intbv(3, min=-5000, max=5000))
    y = Signal(intbv(4, min=-200, max=200))
    intf = IntfWithConstant2()
    inst = top_const(clock, reset, x, y, intf)
    assert inst.analyze_convert() == 0


def test_three_verify():
    inst = c_testbench_three()
    assert inst.verify_convert() == 0



import myhdl
from myhdl import *

def long_divider(
            quotient,
            ready,
            dividend,
            divisor,
            start,
            clock,
            reset
            ):
    

    M = len(dividend)
    N = len(divisor)
    Q = len(quotient)
    
    assert M-N == Q
    
    st = enum("WAIT_START", "CALC")
    
    @instance
    def proc():
        state = st.WAIT_START
        div = intbv(0)[N+1:]
        divbits = intbv(0)[Q-1:]
        quot = intbv(0)[Q:]
        count = intbv(0, min=0, max=Q)
        while True:
            yield clock.posedge, reset.posedge
            if reset == 1:
                quotient.next = 0
                ready.next = 0
                state = st.WAIT_START
                div[:] = 0
                divbits[:] = 0
                quot[:] = 0
            else:
                if state == st.WAIT_START:
                    if start:
                        state = st.CALC
                        ready.next = 0
                        quot[:] = 0
                        div[N+1:] = dividend[M:Q-1]
                        divbits[:] = dividend[Q-1:]
                        count[:] = 0
                        
                elif state == st.CALC:
                    quot[Q:1] = quot[Q-1:]
                    if div >= divisor:
                        quot[0] = 1
                        div -= divisor
                    else:
                        quot[0] = 0
                    
                    if count == Q-1:
                        ready.next = 1
                        state = st.WAIT_START
                    else:
                        div[N+1:1] = div[N:]
                        div[0] = divbits[Q-2]
                        divbits[Q-1:1] = divbits[Q-2:]
                        count += 1
                        
            quotient.next = quot

    return proc


    
if __name__ == '__main__':
    quotient = Signal(intbv(0)[22:])
    ready = Signal(bool())
    dividend = Signal(intbv(0)[38:])
    divisor = Signal(intbv(0)[16:])
    start = Signal(bool())
    clock = Signal(bool())
    reset = Signal(bool())
    
    toVHDL(long_divider,
           quotient,
           ready,
           dividend,
           divisor,
           start,
           clock,
           reset
           )



import os
path = os.path
import random
random.seed(2)

from myhdl import (block, Signal, intbv, instance, delay)

ACTIVE_LOW, INACTIVE_HIGH = bool(0), bool(1)


@block
def bug_1740778 ():
    """ Conversion of min and max attribute.

    """
    s = Signal(intbv(0, min=-13, max=46))

    @instance
    def comb():
        v = intbv(0, min=-15, max=45)
        yield delay(10)
        print(v.min)
        print(v.max)
        print(s.val)
        print(s.min)
        print(s.max)

    return comb


def test_bug_1740778 ():
    assert bug_1740778().verify_convert() == 0




import pytest
from myhdl import Simulation, delay, SimulationError, instance, now, block
from myhdl._Simulation import _error
from helpers import raises_kind


@block
def dut():

    @instance
    def tbstim():
        yield delay(10)
        print("{0:<8d} ".format(now()))
        yield delay(1000)
        print("{0:<8d} ".format(now()))
        for _ in range(10):
            yield delay(1000)

    return tbstim


def issue_104_quit_method():
    sim = Simulation(dut())
    sim.run(1000)
    sim.run(500)
    sim.quit()
    return sim._finished


def issue_104_multiple_instance():
    sim1 = Simulation(dut())
    sim1.run(1000)
    # sim1 is "paused"

    # try and create a second, third, forth simulation instance
    for dummy in range(4):
        with raises_kind(SimulationError, _error.MultipleSim):
            another_sim = Simulation(dut())
    # generating more sims should have failed
    sim1.run(1000)
    sim1.quit()


def test_issue_104():

    assert issue_104_quit_method() == True
    issue_104_multiple_instance()



import os
path = os.path
import unittest
from unittest import TestCase
import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, intbv, delay, instance, toVerilog)
from myhdl._Simulation import Simulation

from .util import setupCosimulation

N = 8
M = 2 ** N
DEPTH = 5


@block
def XorGate(z, a, b, c):

    @instance
    def comb():
        while 1:
            yield a, b, c
            z.next = a ^ b ^ c

    return comb


def randOthers(i, n):
    l = list(range(n))
    l.remove(i)
    random.shuffle(l)
    return l[0], l[1]


@block
def RandomScramblerModule(ol, il, stage=0):
    """ Recursive hierarchy of random xor gates.

    An invented module to check hierarchy with toVerilog.

    """

    sl1 = [Signal(bool()) for __ in range(N)]
    sl2 = [Signal(bool()) for __ in range(N)]
    i1 = [None] * N
    i2 = [None] * N

    if stage < DEPTH:
        for i in range(N):
            j, k = randOthers(i, N)
            i1[i] = XorGate(sl1[i], il[i], il[j], il[k])
        rs = RandomScramblerModule(sl2, sl1, stage=stage + 1)
        for i in range(N):
            j, k = randOthers(i, N)
            i2[i] = XorGate(ol[i], sl2[i], sl2[j], sl2[k])
        return i1, i2, rs
    else:
        for i in range(N):
            j, k = randOthers(i, N)
            i1[i] = XorGate(ol[i], il[i], il[j], il[k])
        return i1


@block
def RandomScrambler(o7, o6, o5, o4, o3, o2, o1, o0,
                    i7, i6, i5, i4, i3, i2, i1, i0):
    sl1 = [i7, i6, i5, i4, i3, i2, i1, i0]
    sl2 = [o7, o6, o5, o4, o3, o2, o1, o0]
    rs = RandomScramblerModule(sl2, sl1, stage=0)
    return rs


o7, o6, o5, o4, o3, o2, o1, o0 = [Signal(bool()) for i in range(N)]
i7, i6, i5, i4, i3, i2, i1, i0 = [Signal(bool()) for i in range(N)]
v7, v6, v5, v4, v3, v2, v1, v0 = [Signal(bool()) for i in range(N)]


def RandomScrambler_v(name,
                      o7, o6, o5, o4, o3, o2, o1, o0,
                      i7, i6, i5, i4, i3, i2, i1, i0):
    return setupCosimulation(**locals())


class TestRandomScrambler(TestCase):

    def stimulus(self):
        tinput = intbv(0)
        output = intbv(0)
        output_v = intbv(0)
        for i in range(100):
        # while 1:
            tinput[:] = randrange(M)
            i7.next = tinput[7]
            i6.next = tinput[6]
            i5.next = tinput[5]
            i4.next = tinput[4]
            i3.next = tinput[3]
            i2.next = tinput[2]
            i1.next = tinput[1]
            i0.next = tinput[0]
            yield delay(10)
            output[7] = o7
            output[6] = o6
            output[5] = o5
            output[4] = o4
            output[3] = o3
            output[2] = o2
            output[1] = o1
            output[0] = o0
            output_v[7] = o7
            output_v[6] = o6
            output_v[5] = o5
            output_v[4] = o4
            output_v[3] = o3
            output_v[2] = o2
            output_v[1] = o1
            output_v[0] = o0
# #             print output
# #             print output_v
# #             print input
            self.assertEqual(output, output_v)

    def test(self):
        rs = toVerilog(RandomScrambler(
                       o7, o6, o5, o4, o3, o2, o1, o0,
                       i7, i6, i5, i4, i3, i2, i1, i0
                       ))
        # time.sleep(1)
        rs_v = RandomScrambler_v(RandomScrambler.__name__,
                                 v7, v6, v5, v4, v3, v2, v1, v0,
                                 i7, i6, i5, i4, i3, i2, i1, i0
                                 )
        sim = Simulation(rs, self.stimulus(), rs_v)
        sim.run()


if __name__ == '__main__':
    unittest.main()




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for Signal """
import copy
import operator
import random
import sys
from random import randrange

import pytest

from myhdl import Signal, intbv
from myhdl._simulator import _siglist

random.seed(1)  # random, but deterministic
maxint = sys.maxsize


class TestSig:

    def setup_method(self, method):
        self.vals = [0, 0, 1, 1, 1, 2, 3, 5, intbv(0), intbv(1), intbv(2)]
        self.nexts = [0, 1, 1, 0, 1, 0, 4, 5, intbv(1), intbv(0), intbv(0)]
        self.vals += [intbv(0), intbv(1), intbv(0), intbv(1), 2           ]
        self.nexts += [intbv(0), intbv(1), 1       , 0       , intbv(3)    ]
        self.vals += [ [1, 2, 3], (1, 2, 3), {1:1, 2:2}, (0, [2, 3], (1, 2))  ]
        self.nexts += [ [4, 5, 6], (4, 5, 5), {3:3, 4:4}, (1, (0, 1), [2, 3])  ]
        self.vals += [bool(0), bool(1), bool(0), bool(1), bool(0), bool(1)]
        self.nexts += [bool(0), bool(1), bool(1), bool(0), 1      , 0      ]
        self.sigs = [Signal(i) for i in self.vals]

        self.incompatibleVals = [ [3, 4], (1, 2), 3 , intbv(0), [1]      ]
        self.incompatibleNexts = [ 4     , 3     , "3", (0)     , intbv(1) ]
        self.incompatibleSigs = [Signal(i) for i in self.incompatibleVals]

        self.eventWaiters = [object() for i in range(3)]
        self.posedgeWaiters = [object() for i in range(5)]
        self.negedgeWaiters = [object() for i in range(7)]

    def testValAttrReadOnly(self):
        """ val attribute should not be writable"""
        s1 = Signal(1)
        with pytest.raises(AttributeError):
            s1.val = 1

    def testDrivenAttrValue(self):
        """ driven attribute only accepts value 'reg' or 'wire' """
        s1 = Signal(1)
        with pytest.raises(ValueError):
            s1.driven = "signal"

    def testPosedgeAttrReadOnly(self):
        """ posedge attribute should not be writable"""
        s1 = Signal(1)
        with pytest.raises(AttributeError):
            s1.posedge = 1

    def testNegedgeAttrReadOnly(self):
        """ negedge attribute should not be writable"""
        s1 = Signal(1)
        with pytest.raises(AttributeError):
            s1.negedge = 1

    def testInitDefault(self):
        """ initial value is None by default """
        s1 = Signal()
        assert s1 == None

    def testInitialization(self):
        """ initial val and next should be equal """
        for s in self.sigs:
            assert s.val == s.next

    def testUpdate(self):
        """ _update() should assign next into val """
        for s, n in zip(self.sigs, self.nexts):
            cur = copy.copy(s.val)
            s.next = n
            # assigning to next should not change current value ...
            assert s.val == cur
            s._update()
            assert s.val == n

    def testNextType(self):
        """ sig.next = n should fail on access if type(n) incompatible """
        i = 0
        for s in (self.sigs + self.incompatibleSigs):
            for n in (self.vals + self.incompatibleVals):
                assert isinstance(s.val, s._type)
                if isinstance(s.val, (int, intbv)):
                    t = (int, intbv)
                else:
                    t = s._type
                if not isinstance(n, t):
                    i += 1
                    with pytest.raises((TypeError, ValueError)):
                        oldval = s.val
                        s.next = n

        assert i >= len(self.incompatibleSigs), "Nothing tested %s" % i

    def testAfterUpdate(self):
        """ updated val and next should be equal but not identical """
        for s, n in zip(self.sigs, self.nexts):
            s.next = n
            s._update()
            assert s.val == s.next

    def testModify(self):
        """ Modifying mutable next should be on a copy """
        for s in self.sigs:
            mutable = 0
            try:
                hash(s.val)
            except TypeError:
                mutable = 1
            if not mutable:
                continue
            if type(s.val) is list:
                s.next.append(1)
            elif type(s.val) is dict:
                s.next[3] = 5
            else:
                s.next  # plain read access
            assert s.val is not s.next, repr(s.val)

    def testUpdatePosedge(self):
        """ update on posedge should return event and posedge waiters """
        s1 = Signal(1)
        s1.next = 0
        s1._update()
        s1.next = 1
        s1._eventWaiters = self.eventWaiters[:]
        s1._posedgeWaiters = self.posedgeWaiters[:]
        s1._negedgeWaiters = self.negedgeWaiters[:]
        waiters = s1._update()
        expected = self.eventWaiters + self.posedgeWaiters
        assert set(waiters) == set(expected)
        assert s1._eventWaiters == []
        assert s1._posedgeWaiters == []
        assert s1._negedgeWaiters == self.negedgeWaiters

    def testUpdateNegedge(self):
        """ update on negedge should return event and negedge waiters """
        s1 = Signal(1)
        s1.next = 1
        s1._update()
        s1.next = 0
        s1._eventWaiters = self.eventWaiters[:]
        s1._posedgeWaiters = self.posedgeWaiters[:]
        s1._negedgeWaiters = self.negedgeWaiters[:]
        waiters = s1._update()
        expected = self.eventWaiters + self.negedgeWaiters
        assert set(waiters) == set(expected)
        assert s1._eventWaiters == []
        assert s1._posedgeWaiters == self.posedgeWaiters
        assert s1._negedgeWaiters == []

    def testUpdateEvent(self):
        """ update on non-edge event should return event waiters """
        s1 = Signal(1)
        s1.next = 4
        s1._update()
        s1.next = 5
        s1._eventWaiters = self.eventWaiters[:]
        s1._posedgeWaiters = self.posedgeWaiters[:]
        s1._negedgeWaiters = self.negedgeWaiters[:]
        waiters = s1._update()
        expected = self.eventWaiters
        assert set(waiters) == set(expected)
        assert s1._eventWaiters == []
        assert s1._posedgeWaiters == self.posedgeWaiters
        assert s1._negedgeWaiters == self.negedgeWaiters

    def testUpdateNoEvent(self):
        """ update without value change should not return event waiters """
        s1 = Signal(1)
        s1.next = 4
        s1._update()
        s1.next = 4
        s1._eventWaiters = self.eventWaiters[:]
        s1._posedgeWaiters = self.posedgeWaiters[:]
        s1._negedgeWaiters = self.negedgeWaiters[:]
        waiters = s1._update()
        assert waiters == []
        assert s1._eventWaiters == self.eventWaiters
        assert s1._posedgeWaiters == self.posedgeWaiters
        assert s1._negedgeWaiters == self.negedgeWaiters

    def testNextAccess(self):
        """ each next attribute access puts a sig in a global siglist """
        del _siglist[:]
        s = [None] * 4
        for i in range(len(s)):
            s[i] = Signal(i)
        s[1].next  # read access
        s[2].next = 1
        s[2].next
        s[3].next = 0
        s[3].next = 1
        s[3].next = 3
        for i in range(len(s)):
            assert _siglist.count(s[i]) == i


class TestSignalAsNum:

    def seqSetup(self, imin, imax, jmin=0, jmax=None):
        seqi = [imin, imin, 12, 34]
        seqj = [jmin, 12  , jmin, 34]
        if not imax and not jmax:
            l = 2222222222222222222222222222
            seqi.append(l)
            seqj.append(l)
        # first some smaller ints
        for __ in range(100):
            ifirstmax = jfirstmax = 100000
            if imax:
                ifirstmax = min(imax, ifirstmax)
            if jmax:
                jfirstmax = min(jmax, jfirstmax)
            i = randrange(imin, ifirstmax)
            j = randrange(jmin, jfirstmax)
            seqi.append(i)
            seqj.append(j)
        # then some potentially longs
        for __ in range(100):
            if not imax:
                i = randrange(maxint) + randrange(maxint)
            else:
                i = randrange(imin, imax)
            if not jmax:
                j = randrange(maxint) + randrange(maxint)
            else:
                j = randrange(jmin, jmax)
            seqi.append(i)
            seqj.append(j)
        self.seqi = seqi
        self.seqj = seqj

    def binaryCheck(self, op, imin=0, imax=None, jmin=0, jmax=None):
        self.seqSetup(imin=imin, imax=imax, jmin=jmin, jmax=jmax)
        for i, j in zip(self.seqi, self.seqj):
            bi = Signal(int(i))
            bj = Signal(int(j))
            ref = op(int(i), j)
            r1 = op(bi, j)
            r2 = op(int(i), bj)
            r3 = op(bi, bj)
            assert type(r1) == type(ref)
            assert type(r2) == type(ref)
            assert type(r3) == type(ref)
            assert r1 == ref
            assert r2 == ref
            assert r3 == ref

    def augmentedAssignCheck(self, op, imin=0, imax=None, jmin=0, jmax=None):
        self.seqSetup(imin=imin, imax=imax, jmin=jmin, jmax=jmax)
        for i, j in zip(self.seqi, self.seqj):
            bj = Signal(j)
            ref = int(i)
            ref = op(ref, j)
            r1 = bi1 = Signal(i)
            with pytest.raises(TypeError):
                r1 = op(r1, j)

            r2 = int(i)
            r2 = op(r2, bj)
            r3 = bi3 = Signal(i)
            with pytest.raises(TypeError):
                r3 = op(r3, bj)

            assert r2 == ref

    def unaryCheck(self, op, imin=0, imax=None):
        self.seqSetup(imin=imin, imax=imax)
        for i in self.seqi:
            bi = Signal(i)
            ref = op(i)
            r1 = op(bi)
            assert type(r1) == type(ref)
            assert r1 == ref

    def conversionCheck(self, op, imin=0, imax=None):
        self.seqSetup(imin=imin, imax=imax)
        for i in self.seqi:
            bi = Signal(i)
            ref = op(i)
            r1 = op(bi)
            assert type(r1) == type(ref)
            assert r1 == ref

    def comparisonCheck(self, op, imin=0, imax=None, jmin=0, jmax=None):
        self.seqSetup(imin=imin, imax=imax, jmin=jmin, jmax=jmax)
        for i, j in zip(self.seqi, self.seqj):
            bi = Signal(i)
            bj = Signal(j)
            ref = op(i, j)
            r1 = op(bi, j)
            r2 = op(i, bj)
            r3 = op(bi, bj)
            assert r1 == ref
            assert r2 == ref
            assert r3 == ref

    def testAdd(self):
        self.binaryCheck(operator.add)

    def testSub(self):
        self.binaryCheck(operator.sub)

    def testMul(self):
        self.binaryCheck(operator.mul, imax=maxint)  # XXX doesn't work for long i???

    def testFloorDiv(self):
        self.binaryCheck(operator.floordiv, jmin=1)

    def testMod(self):
        self.binaryCheck(operator.mod, jmin=1)

    def testPow(self):
        self.binaryCheck(pow, jmax=64)

    def testLShift(self):
        self.binaryCheck(operator.lshift, jmax=256)

    def testRShift(self):
        self.binaryCheck(operator.rshift, jmax=256)

    def testAnd(self):
        self.binaryCheck(operator.and_)

    def testOr(self):
        self.binaryCheck(operator.or_)

    def testXor(self):
        self.binaryCheck(operator.xor)

    def testIAdd(self):
        self.augmentedAssignCheck(operator.iadd)

    def testISub(self):
        self.augmentedAssignCheck(operator.isub)

    def testIMul(self):
        self.augmentedAssignCheck(operator.imul, imax=maxint)  # XXX doesn't work for long i???

    def testIFloorDiv(self):
        self.augmentedAssignCheck(operator.ifloordiv, jmin=1)

    def testIMod(self):
        self.augmentedAssignCheck(operator.imod, jmin=1)

    def testIPow(self):
        self.augmentedAssignCheck(operator.ipow, jmax=64)

    def testIAnd(self):
        self.augmentedAssignCheck(operator.iand)

    def testIOr(self):
        self.augmentedAssignCheck(operator.ior)

    def testIXor(self):
        self.augmentedAssignCheck(operator.ixor)

    def testILShift(self):
        self.augmentedAssignCheck(operator.ilshift, jmax=256)

    def testIRShift(self):
        self.augmentedAssignCheck(operator.irshift, jmax=256)

    def testNeg(self):
        self.unaryCheck(operator.neg)

    def testPos(self):
        self.unaryCheck(operator.pos)

    def testAbs(self):
        self.unaryCheck(operator.abs)

    def testInvert(self):
        self.unaryCheck(operator.inv)

    def testInt(self):
        self.conversionCheck(int, imax=maxint)

    def testLong(self):
        self.conversionCheck(int)

    def testFloat(self):
        self.conversionCheck(float)

    # XXX __complex__ seems redundant ??? (complex() works as such?)

    def testOct(self):
        self.conversionCheck(oct)

    def testHex(self):
        self.conversionCheck(hex)

    def testLt(self):
        self.comparisonCheck(operator.lt)

    def testLe(self):
        self.comparisonCheck(operator.le)

    def testGt(self):
        self.comparisonCheck(operator.gt)

    def testGe(self):
        self.comparisonCheck(operator.ge)

    def testEq(self):
        self.comparisonCheck(operator.eq)

    def testNe(self):
        self.comparisonCheck(operator.ne)


def getItem(s, i):
    ext = '0' * (i - len(s) + 1)
    exts = ext + s
    si = len(exts) - 1 - i
    return exts[si]


def getSlice(s, i, j):
    ext = '0' * (i - len(s) + 1)
    exts = ext + s
    si = len(exts) - i
    sj = len(exts) - j
    return exts[si:sj]


class TestSignalIntBvIndexing:

    def seqsSetup(self):
        seqs = ["0", "1", "000", "111", "010001", "110010010", "011010001110010"]
        seqs.extend(["0101010101", "1010101010", "00000000000", "11111111111111"])
        seqs.append("11100101001001101000101011011101001101")
        seqs.append("00101011101001011111010100010100100101010001001")
        self.seqs = seqs
        seqv = ["0", "1", "10", "101", "1111", "1010"]
        seqv.extend(["11001", "00111010", "100111100"])
        seqv.append("0110101001111010101110011010011")
        seqv.append("1101101010101101010101011001101101001100110011")
        self.seqv = seqv

    def testGetItem(self):
        self.seqsSetup()
        for s in self.seqs:
            n = int(s, 2)
            sbv = Signal(intbv(n))
            sbvi = Signal(intbv(~n))
            for i in range(len(s) + 20):
                ref = int(getItem(s, i), 2)
                res = sbv[i]
                resi = sbvi[i]
                assert res == ref
                assert type(res) == bool
                assert resi == ref ^ 1
                assert type(resi) == bool

    def testGetSlice(self):
        self.seqsSetup()
        for s in self.seqs:
            n = int(s, 2)
            sbv = Signal(intbv(n))
            sbvi = Signal(intbv(~n))
            for i in range(1, len(s) + 20):
                for j in range(0, len(s) + 20):
                    try:
                        res = sbv[i:j]
                        resi = sbvi[i:j]
                    except ValueError:
                        assert i <= j
                        continue
                    ref = int(getSlice(s, i, j), 2)
                    assert res == ref
                    assert type(res) == intbv
                    mask = (2 ** (i - j)) - 1
                    assert resi == ref ^ mask
                    assert type(resi) == intbv

    def testSetItem(self):
        sbv = Signal(intbv(5))
        with pytest.raises(TypeError):
            sbv[1] = 1

    def testSetSlice(self):
        sbv = Signal(intbv(5))
        with pytest.raises(TypeError):
            sbv[1:0] = 1


class TestSignalNrBits:

    def testBool(self):
        if type(bool) is not type:  # bool not a type in 2.2
            return
        s = Signal(bool())
        assert s._nrbits == 1

    def testIntbvSlice(self):
        for n in range(1, 40):
            for m in range(0, n):
                s = Signal(intbv()[n:m])
                assert s._nrbits == n - m

    def testIntbvBounds(self):
        for n in range(1, 40):
            s = Signal(intbv(min=-(2 ** n)))
            assert s._nrbits == 0
            s = Signal(intbv(max=2 ** n))
            assert s._nrbits == 0
            s = Signal(intbv(min=0, max=2 ** n))
            assert s._nrbits == n
            s = Signal(intbv(1, min=1, max=2 ** n))
            assert s._nrbits == n
            s = Signal(intbv(min=0, max=2 ** n + 1))
            assert s._nrbits == n + 1
            s = Signal(intbv(min=-(2 ** n), max=2 ** n - 1))
            assert s._nrbits == n + 1
            s = Signal(intbv(min=-(2 ** n), max=1))
            assert s._nrbits == n + 1
            s = Signal(intbv(min=-(2 ** n) - 1, max=2 ** n - 1))
            assert s._nrbits == n + 2


class TestSignalBoolBounds:

    def testSignalBoolBounds(self):
        if type(bool) is not type:  # bool not a type in 2.2
            return
        s = Signal(bool())
        s.next = 1
        s.next = 0
        for v in (-1, -8, 2, 5):
            with pytest.raises(ValueError):
                s.next = v


class TestSignalIntbvBounds:

    def testSliceAssign(self):
        s = Signal(intbv(min=-24, max=34))
        for i in (-24, -2, 13, 33):
            for k in (6, 9, 10):
                s.next[:] = 0
                s.next[k:] = i
                assert s.next == i
        for i in (-25, -128, 34, 35, 229):
            for k in (0, 9, 10):
                with pytest.raises(ValueError):
                    s.next[k:] = i

        s = Signal(intbv(5)[8:])
        for v in (0, 2 ** 8 - 1, 100):
            s.next[:] = v
        for v in (-1, 2 ** 8, -10, 1000):
            with pytest.raises(ValueError):
                s.next[:] = v



import pytest


class raises_kind(object):
    def __init__(self, exc, kind):
        self.exc = exc
        self.kind = kind

    def __enter__(self):
        return None

    def __exit__(self, *tp):
        __tracebackhide__ = True
        if tp[0] is None:
            pytest.fail("DID NOT RAISE")
        assert tp[1].kind == self.kind
        return issubclass(tp[0], self.exc)



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2013 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" MyHDL conversion analysis module.

"""
# import compiler
# from compiler import ast as astNode
from types import FunctionType, MethodType
import sys
import re
import ast
import builtins
from itertools import chain

import myhdl
from myhdl import *
from myhdl import ConversionError
from myhdl._always_comb import _AlwaysComb
from myhdl._always_seq import _AlwaysSeq
from myhdl._always import _Always
from myhdl.conversion._misc import (_error, _access, _kind,
                                    _ConversionMixin, _Label, _genUniqueSuffix,
                                    _get_argnames)
from myhdl._extractHierarchy import _isMem, _getMemInfo, _UserCode
from myhdl._Signal import _Signal, _WaiterList, _isListOfSigs
from myhdl._ShadowSignal import _ShadowSignal, _SliceSignal, _TristateDriver
from myhdl._util import _flatten
from myhdl._util import _isTupleOfInts
from myhdl._util import _makeAST
from myhdl._resolverefs import _AttrRefTransformer
from myhdl._misc import isboundmethod

from myhdl._hdlclass import HdlClass

myhdlObjects = myhdl.__dict__.values()
builtinObjects = builtins.__dict__.values()

_enumTypeSet = set()


def _makeName(n, prefixes, namedict):
    # trim empty prefixes
    prefixes = [p for p in prefixes if p]
    if len(prefixes) > 1:
        #        name = '_' + '_'.join(prefixes[1:]) + '_' + n
        name = '_'.join(prefixes[1:]) + '_' + n
    else:
        name = n
    if '[' in name or ']' in name:
        name = "\\" + name + ' '

    # dirty fix (for class based designs): remove 'self_' if there
    return name.replace('self_', '')


def _analyzeSigs(hierarchy, hdl='Verilog'):
    curlevel = 0
    siglist = []
    memlist = []
    prefixes = []

    open, close = '[', ']'
    if hdl == 'VHDL':
        open, close = '(', ')'

    for inst in hierarchy:
        level = inst.level
        name = inst.name
        sigdict = inst.sigdict
        memdict = inst.memdict
        namedict = dict(chain(sigdict.items(), memdict.items()))
        delta = curlevel - level
        curlevel = level
        assert(delta >= -1)
        if delta > -1: # same or higher level
            prefixes = prefixes[:curlevel - 1]
        # skip processing and prefixing in context without signals
        # if not (sigdict or memdict):
        #    prefixes.append("")
        #    continue
        prefixes.append(name)
        for n, s in sigdict.items():
            if s._name is not None:
                continue
            if isinstance(s, _SliceSignal):
                continue
            s._name = _makeName(n, prefixes, namedict)
            if not s._nrbits:
                raise ConversionError(_error.UndefinedBitWidth, s._name)
            # slice signals
            for sl in s._slicesigs:
                sl._setName(hdl)
            siglist.append(s)

        # list of signals
        for n, m in memdict.items():
            if m.name is not None:
                continue
            m.name = _makeName(n, prefixes, namedict)
            memlist.append(m)

    # handle the case where a named signal appears in a list also by giving
    # priority to the list and marking the signals as unused
    for m in memlist:
        if not m._used:
            continue
        for i, s in enumerate(m.mem):
            s._name = "%s%s%s%s" % (m.name, open, i, close)
            s._used = False
            if s._inList:
                raise ConversionError(_error.SignalInMultipleLists, s._name)
            s._inList = True
            if not s._nrbits:
                raise ConversionError(_error.UndefinedBitWidth, s._name)
            if type(s.val) != type(m.elObj.val):
                raise ConversionError(_error.InconsistentType, s._name)
            if s._nrbits != m.elObj._nrbits:
                raise ConversionError(_error.InconsistentBitWidth, s._name)

    return siglist, memlist


def _analyzeGens(top, absnames):
    genlist = []
    for g in top:
        if isinstance(g, _UserCode):
            tree = g
        elif isinstance(g, (_AlwaysComb, _AlwaysSeq, _Always)):
            f = g.func
            tree = g.ast
            tree.symdict = f.__globals__.copy()
            tree.callstack = []
            # handle free variables
            tree.nonlocaldict = {}
            if f.__code__.co_freevars:
                for n, c in zip(f.__code__.co_freevars, f.__closure__):
                    obj = c.cell_contents
                    tree.symdict[n] = obj
                    # currently, only intbv as automatic nonlocals (until Python 3.0)
                    if isinstance(obj, intbv):
                        tree.nonlocaldict[n] = obj
            # tree.name = absnames.get(id(g), str(_Label("BLOCK"))).upper()
            tree.name = absnames.get(id(g), str(_Label("BLOCK"))).lower()
            v = _AttrRefTransformer(tree)
            v.visit(tree)
            v = _FirstPassVisitor(tree)
            v.visit(tree)
            if isinstance(g, _AlwaysComb):
                v = _AnalyzeAlwaysCombVisitor(tree, g.senslist)
            elif isinstance(g, _AlwaysSeq):
                v = _AnalyzeAlwaysSeqVisitor(tree, g.senslist, g.reset, g.sigregs, g.varregs)
            else:
                v = _AnalyzeAlwaysDecoVisitor(tree, g.senslist)
            v.visit(tree)
        else: # @instance
            f = g.gen.gi_frame
            tree = g.ast
            tree.symdict = f.f_globals.copy()
            tree.symdict.update(f.f_locals)
            tree.nonlocaldict = {}
            tree.callstack = []
            # tree.name = absnames.get(id(g), str(_Label("BLOCK"))).upper()
            tree.name = absnames.get(id(g), str(_Label("BLOCK"))).lower()
            v = _AttrRefTransformer(tree)
            v.visit(tree)
            v = _FirstPassVisitor(tree)
            v.visit(tree)
            v = _AnalyzeBlockVisitor(tree)
            v.visit(tree)
        genlist.append(tree)
    return genlist


class _FirstPassVisitor(ast.NodeVisitor, _ConversionMixin):

    """First pass visitor.

    Prune unsupported contructs, and add some useful attributes.

    """

    def __init__(self, tree):
        self.tree = tree
        self.toplevel = True

    def visit_Tuple(self, node):
        if isinstance(node.ctx, ast.Store):
            self.raiseError(node, _error.NotSupported, "tuple assignment")

    def visit_Repr(self, node):
        self.raiseError(node, _error.NotSupported, "backquote")

    def visit_ClassDef(self, node):
        self.raiseError(node, _error.NotSupported, "class statement")

    def visit_Dict(self, node):
        self.raiseError(node, _error.NotSupported, "dictionary")

    def visit_BinOp(self, node):
        if isinstance(node.op, ast.Div):
            self.raiseError(node, _error.NotSupported, "true division - consider '//'")

    def visit_Ellipsis(self, node):
        self.raiseError(node, _error.NotSupported, "ellipsis")

    def visit_Exec(self, node):
        self.raiseError(node, _error.NotSupported, "exec statement")

    def visitExpression(self, node, *args):
        self.raiseError(node, _error.NotSupported, "Expression node")

    def visit_ImportFrom(self, node):
        self.raiseError(node, _error.NotSupported, "from statement")

    def visit_Global(self, node):
        self.raiseError(node, _error.NotSupported, "global statement")

    def visit_Import(self, node):
        self.raiseError(node, _error.NotSupported, "import statement")

    def visit_Lambda(self, node):
        self.raiseError(node, _error.NotSupported, "lambda statement")

    def visit_ListComp(self, node):
        if len(node.generators) > 1:
            self.raiseError(node, _error.NotSupported,
                            "multiple for statements in list comprehension")
        if node.generators[0].ifs:
            self.raiseError(node, _error.NotSupported, "if statement in list comprehension")
        self.generic_visit(node)

    def visit_List(self, node):
        self.raiseError(node, _error.NotSupported, "list")

    def visitSliceObj(self, node):
        self.raiseError(node, _error.NotSupported, "slice object")

    # All try blocks from python 3.3+
    def visit_Try(self, node):
        self.raiseError(node, _error.NotSupported, "try statement")

    # Legacy try blocks
    def visit_TryExcept(self, node):
        self.raiseError(node, _error.NotSupported, "try-except statement")

    def visit_TryFinally(self, node):
        self.raiseError(node, _error.NotSupported, "try-finally statement")

    def visit_Assign(self, node):
        if len(node.targets) > 1:
            self.raiseError(node, _error.NotSupported, "multiple assignments")
        self.visit(node.targets[0])
        self.visit(node.value)

    def visit_Call(self, node):
        # ast.Call signature changed in python 3.5
        # http://greentreesnakes.readthedocs.org/en/latest/nodes.html#Call
        starargs = any(isinstance(arg, ast.Starred) for arg in node.args)
        kwargs = any(kw.arg is None for kw in node.keywords)

        if starargs:
            self.raiseError(node, _error.NotSupported, "extra positional arguments")
        if kwargs:
            self.raiseError(node, _error.NotSupported, "extra named arguments")
        self.generic_visit(node)

    def visit_Compare(self, node):
        if len(node.ops) != 1:
            self.raiseError(node, _error.NotSupported, "chained comparison")
        self.generic_visit(node)

    def visit_FunctionDef(self, node):
        if node.args.vararg or node.args.kwarg:
            self.raiseError(node, _error.NotSupported, "extra positional or named arguments")
        if not self.toplevel:
            self.raiseError(node, _error.NotSupported, "embedded function definition")
        self.toplevel = False
        node.argnames = _get_argnames(node)
        # don't visit decorator lists - they can support more than other calls
        # put official docstrings aside for separate processing
        node.doc = None
        if node.body and isinstance(node.body[0], ast.Expr) and \
            isinstance(node.body[0].value, ast.Constant) and isinstance(node.body[0].value.value, str):
            node.doc = node.body[0].value.value
            node.body = node.body[1:]
        self.visitList(node.body)

    def flattenIf(self, node, tests, else_, co):
        """ Flatten if-then-else as in compiler package."""
        if node:
            if len(node) == 1 and \
                    isinstance(node[0], ast.If) and \
                    node[0].body[0].col_offset == co: # ugly hack to detect separate else clause
                elifnode = node[0]
                tests.append((elifnode.test, elifnode.body))
                self.flattenIf(elifnode.orelse, tests, else_, co)
            else:
                else_[:] = node

    def visit_If(self, node):
        node.ignore = False
        if not node.orelse:
            test = node.test
            if isinstance(test, ast.Name):
                if test.id == '__debug__':
                    node.ignore = True
                    return  # skip
        self.generic_visit(node)

        # add fields that match old compiler package
        tests = [(node.test, node.body)]
        else_ = []
        self.flattenIf(node.orelse, tests, else_, node.body[0].col_offset)
        node.tests = tests
        node.else_ = else_

    def visit_Print(self, node):
        if node.dest is not None:
            self.raiseError(node, _error.NotSupported, "printing to a file with >> syntax")
        if not node.nl:
            self.raiseError(node, _error.NotSupported, "printing without newline")


def getNrBits(obj):
    if hasattr(obj, '_nrbits'):
        return obj._nrbits
    return None


def hasType(obj, theType):
    if isinstance(obj, theType):
        return True
    if isinstance(obj, _Signal):
        if isinstance(obj._val, theType):
            return True
    return False


class ReferenceStack(list):

    def push(self):
        self.append(set())

    def add(self, item):
        self[-1].add(item)

    def __contains__(self, item):
        for s in self:
            if item in s:
                return True
        return False


class _Ram(object):
    __slots__ = ['elObj', 'depth']


class _Rom(object):
    __slots__ = ['rom']

    def __init__(self, rom):
        self.rom = rom


re_str = re.compile(r"[^%]+")
re_ConvSpec = re.compile(r"%(?P<justified>[-]?)(?P<width>[0-9]*)(?P<conv>[sd])")


class ConvSpec(object):

    def __init__(self, **kwargs):
        self.justified = "RIGHT"
        self.width = 0
        self.conv = str
        if kwargs['justified'] == '-':
            self.justified = "LEFT"
        if kwargs['width']:
            self.width = int(kwargs['width'])
        if kwargs['conv'] == 'd':
            self.conv = int


defaultConvSpec = ConvSpec(**re_ConvSpec.match(r"%s").groupdict())


def _getNritems(obj):
    """Return the number of items in an objects' type"""
    if isinstance(obj, _Signal):
        obj = obj._init
    if isinstance(obj, intbv):
        return obj._max - obj._min
    elif isinstance(obj, EnumItemType):
        return len(obj._type)
    else:
        raise TypeError("Unexpected type, missing final \'else:\'?")


class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):

    def __init__(self, tree):
        tree.sigdict = {}
        tree.vardict = {}
        tree.inputs = set()
        tree.outputs = set()
        # hack for assigned mems
        tree.outmems = set()
        tree.argnames = []
        tree.kind = None
        tree.hasYield = 0
        tree.hasRom = False
        tree.hasLos = False
        tree.hasPrint = False
        self.tree = tree
        self.labelStack = []
        self.refStack = ReferenceStack()
        self.globalRefs = set()
        self.access = _access.INPUT
        self.kind = _kind.NORMAL

    def visit_BinOp(self, node):
        self.visit(node.left)
        self.visit(node.right)
        node.obj = int(-1)

    def visit_BoolOp(self, node):
        for n in node.values:
            self.visit(n)
        for n in node.values:
            if not hasType(n.obj, bool):
                self.raiseError(node, _error.NotSupported,
                                "non-boolean argument in logical operator")
        node.obj = bool()

    def visit_UnaryOp(self, node):
        self.visit(node.operand)
        op = node.op
        node.obj = node.operand.obj
        if isinstance(op, ast.Not):
            node.obj = bool()
        elif isinstance(op, ast.UAdd):
            node.obj = int(-1)
        elif isinstance(op, ast.USub):
            node.obj = int(-1)

    def visit_Attribute(self, node):
        if isinstance(node.ctx, ast.Store):
            self.setAttr(node)
        else:
            self.getAttr(node)
        if node.attr == 'next':
            if isinstance(node.value, ast.Name):
                n = node.value.id
                obj = node.value.obj
                if isinstance(obj, _Signal) and isinstance(obj._init, modbv):
                    if not obj._init._hasFullRange():
                        self.raiseError(node, _error.ModbvRange, n)

    def setAttr(self, node):
        if node.attr != 'next':
            self.raiseError(node, _error.NotSupported, "attribute assignment")
        self.tree.kind = _kind.TASK
        # self.access = _access.OUTPUT
        self.visit(node.value)
        node.obj = node.value.obj
        # self.access = _access.INPUT

    def getAttr(self, node):
        self.visit(node.value)
        node.obj = None
        if isinstance(node.value, ast.Name):
            n = node.value.id
            if (n not in self.tree.vardict) and (n not in self.tree.symdict):
                raise AssertionError("attribute target: %s" % n)
        obj = node.value.obj
        if isinstance(obj, _Signal):
            if node.attr == 'posedge':
                node.obj = obj.posedge
            elif node.attr == 'negedge':
                node.obj = obj.negedge
            elif node.attr in ('val', 'next'):
                node.obj = obj.val
        if isinstance(obj, (intbv, _Signal)):
            if node.attr == 'min':
                node.obj = obj.min
            elif node.attr == 'max':
                node.obj = obj.max
            elif node.attr == 'signed':
                node.obj = intbv.signed
        if isinstance(obj, EnumType):
            assert hasattr(obj, node.attr), node.attr
            node.obj = getattr(obj, node.attr)
            if obj not in _enumTypeSet:
                _enumTypeSet.add(obj)
                suf = _genUniqueSuffix.next()
                obj._setName(n + suf)
        if node.obj is None: # attribute lookup failed
            self.raiseError(node, _error.UnsupportedAttribute, node.attr)

    def visit_Assign(self, node):
        target, value = node.targets[0], node.value
        self.access = _access.OUTPUT
        self.visit(target)
        self.access = _access.INPUT
        # set attribute to detect a top-level rhs
        value.isRhs = True
        if isinstance(target, ast.Name):
            node.kind = _kind.DECLARATION
            self.kind = _kind.DECLARATION
            self.visit(value)
            self.kind = _kind.NORMAL
            n = target.id
            if n in self.tree.sigdict:
                self.raiseError(node, _error.ShadowingVar)
            obj = self.getObj(value)
            if obj is None:
                self.raiseError(node, _error.TypeInfer, n)
            if isinstance(obj, intbv):
                if len(obj) == 0:
                    self.raiseError(node, _error.IntbvBitWidth, n)
            if isinstance(obj, modbv):
                if not obj._hasFullRange():
                    self.raiseError(node, _error.ModbvRange, n)
            if n in self.tree.vardict:
                curObj = self.tree.vardict[n]
                if isinstance(obj, type(curObj)):
                    pass
                elif isinstance(curObj, type(obj)):
                    self.tree.vardict[n] = obj
                else:
                    self.raiseError(node, _error.TypeMismatch, n)
                if getNrBits(obj) != getNrBits(curObj):
                    self.raiseError(node, _error.NrBitsMismatch, n)
            else:
                self.tree.vardict[n] = obj
        else:
            self.visit(value)

    def visit_AugAssign(self, node):
        # declare node as an rhs for type inference optimization
        node.isRhs = True
        self.access = _access.INOUT
        self.visit(node.target)
        self.access = _access.INPUT
        self.visit(node.value)

    def visit_Break(self, node):
        self.labelStack[-2].isActive = True

    def visit_Call(self, node):
        self.visit(node.func)
        f = self.getObj(node.func)
        node.obj = None

        if f is print:
            self.visit_Print(node)
            return

        self.access = _access.UNKNOWN
        for arg in node.args:
            self.visit(arg)
        for kw in node.keywords:
            self.visit(kw)
        self.access = _access.INPUT
        argsAreInputs = True
        if type(f) is type and issubclass(f, intbv):
            node.obj = self.getVal(node)
        elif f is concat:
            node.obj = self.getVal(node)
        elif f is len:
            self.access = _access.UNKNOWN
            node.obj = int(0)  # XXX
        elif f is bool:
            node.obj = bool()
        elif f is int:
            node.obj = int(-1)
# elif f in (posedge , negedge):
# #             node.obj = _EdgeDetector()
        elif f is ord:
            node.obj = int(-1)
            if isinstance(node.args[0], ast.Constant) and \
               isinstance(node.args[0].value, str) and \
               (len(node.args[0].value) == 1):
                pass
            else:
                self.raiseError(node, _error.NotSupported,
                                "ord: expect string argument with length 1")
        elif f is delay:
            node.obj = delay(0)
        # suprize: identity comparison on unbound methods doesn't work in python 2.5??
        elif f == intbv.signed:
            obj = node.func.value.obj
            if len(obj):
                M = 2 ** (len(obj) - 1)
                node.obj = intbv(-1, min=-M, max=M)
            else:
                node.obj = intbv(-1)
        elif f in myhdlObjects:
            pass
        elif f in builtinObjects:
            pass
        elif type(f) is FunctionType:
            argsAreInputs = False
            tree = _makeAST(f)
            fname = f.__name__
            tree.name = _Label(fname)
            tree.symdict = f.__globals__.copy()
            tree.nonlocaldict = {}
            if fname in self.tree.callstack:
                self.raiseError(node, _error.NotSupported, "Recursive call")
            tree.callstack = self.tree.callstack[:]
            tree.callstack.append(fname)
            # handle free variables
            if f.__code__.co_freevars:
                for n, c in zip(f.__code__.co_freevars, f.__closure__):
                    obj = c.cell_contents
                    if not isinstance(obj, (int, _Signal)):
                        self.raiseError(node, _error.FreeVarTypeError, n)
                    tree.symdict[n] = obj
            v = _FirstPassVisitor(tree)
            v.visit(tree)
            v = _AnalyzeFuncVisitor(tree, node.args, node.keywords)
            v.visit(tree)
            node.obj = tree.returnObj
            node.tree = tree
            tree.argnames = argnames = _get_argnames(tree.body[0])
            # extend argument list with keyword arguments on the correct position
            node.args.extend([None] * len(node.keywords))
            for kw in node.keywords:
                node.args[argnames.index(kw.arg)] = kw.value
            for n, arg in zip(argnames, node.args):
                if n in tree.outputs:
                    self.access = _access.OUTPUT
                    self.visit(arg)
                    self.access = _access.INPUT
                if n in tree.inputs:
                    self.visit(arg)
        elif type(f) is MethodType:
            self.raiseError(node, _error.NotSupported, "method call: '%s'" % f.__name__)
        else:
            debug_info = [e for e in ast.iter_fields(node.func)]
            raise AssertionError("Unexpected callable %s" % str(debug_info))
        if argsAreInputs:
            for arg in node.args:
                self.visit(arg)

    def visit_Compare(self, node):
        node.obj = bool()
        for n in [node.left] + node.comparators:
            self.visit(n)
        op, arg = node.ops[0], node.comparators[0]
# #         node.expr.target = self.getObj(arg)
# #         arg.target = self.getObj(node.expr)
        # detect specialized case for the test
        if isinstance(op, ast.Eq) and isinstance(node.left, ast.Name):
            # check wether it can be a case
            val = arg.obj
            if isinstance(val, bool):
                val = int(val)  # cast bool to int first
            if isinstance(val, (EnumItemType, int)):
                node.case = (node.left, val)
            # check whether it can be part of an edge check
            n = node.left.id
            if n in self.tree.sigdict:
                sig = self.tree.sigdict[n]
                v = self.getValue(arg)
                if v is not None:
                    if v == 0:
                        node.edge = sig.negedge
                    elif v == 1:
                        node.edge = sig.posedge

    def visit_Constant(self, node):
        node.obj = None  # safeguarding?
        # ToDo check for tuples?
        if isinstance(node.value, int):
            # Num
            if node.value in (0, 1):
                node.obj = bool(node.value)
            else:
                node.obj = node.value
        elif node.value in (True, False, None):
            # NameConstant
            node.obj = node.value
        elif isinstance(node.value, str):
            # Str
            node.obj = node.value

    def visit_Continue(self, node):
        self.labelStack[-1].isActive = True

    def visit_For(self, node):
        node.breakLabel = _Label("BREAK")
        node.loopLabel = _Label("LOOP")
        self.labelStack.append(node.breakLabel)
        self.labelStack.append(node.loopLabel)
        self.refStack.push()
        self.visit(node.target)
        var = node.target.id
        self.tree.vardict[var] = int(-1)

        cf = node.iter
        self.visit(cf)
        self.require(node, isinstance(cf, ast.Call), "Expected (down)range call")
        f = self.getObj(cf.func)
        self.require(node, f in (range, downrange), "Expected (down)range call")

        for stmt in node.body:
            self.visit(stmt)
        self.refStack.pop()
        self.require(node, not node.orelse, "for-else not supported")
        self.labelStack.pop()
        self.labelStack.pop()

    def visit_FunctionDef(self, node):
        raise AssertionError("subclass must implement this")

    def visit_If(self, node):
        if node.ignore:
            return
        for test, suite in node.tests:
            self.visit(test)
            self.refStack.push()
            self.visitList(suite)
            self.refStack.pop()
        if node.else_:
            self.refStack.push()
            self.visitList(node.else_)
            self.refStack.pop()
        # check whether the if can be mapped to a (parallel) case
        node.isCase = node.isFullCase = False
        test1 = node.tests[0][0]
        if not hasattr(test1, 'case'):
            return
        var1, item1 = test1.case
        # don't infer a case if there's no elsif test
        if not node.tests[1:]:
            return
        choices = set()
        choices.add(item1)

        for test, suite in node.tests[1:]:
            if not hasattr(test, 'case'):
                return
            var, item = test.case
            if var.id != var1.id or type(item) is not type(item1):
                return
            if item in choices:
                return
            choices.add(item)
        node.isCase = True
        node.caseVar = var1
        node.caseItem = item1
        if node.else_ or (len(choices) == _getNritems(var1.obj)):
            node.isFullCase = True

    def visit_ListComp(self, node):
        mem = node.obj = _Ram()
        self.kind = _kind.DECLARATION
        try:
            self.visit(node.elt)
        except ConversionError as e:
            if e.kind == _error.UnboundLocal:
                pass
            else:
                raise
        self.kind = _kind.NORMAL
        mem.elObj = self.getObj(node.elt)
        if not isinstance(mem.elObj, intbv) or not len(mem.elObj) > 0:
            self.raiseError(node, _error.UnsupportedListComp)
        cf = node.generators[0].iter
        self.visit(cf)
        if not isinstance(cf, ast.Call):
            self.raiseError(node, _error.UnsupportedListComp)
        f = self.getObj(cf.func)
        if f is not range or len(cf.args) != 1:
            self.raiseError(node, _error.UnsupportedListComp)
        mem.depth = cf.args[0].obj

    def visit_Name(self, node):
        if isinstance(node.ctx, ast.Store):
            self.setName(node)
        else:
            self.getName(node)

    def setName(self, node):
        # XXX INOUT access in Store context, unlike with compiler
        # XXX check whether ast context is correct
        n = node.id
        if self.access == _access.INOUT: # augmented assign
            if n in self.tree.sigdict:
                sig = self.tree.sigdict[n]
                if isinstance(sig, _Signal):
                    self.raiseError(node, _error.NotSupported, "Augmented signal assignment")
            if n in self.tree.vardict:
                obj = self.tree.vardict[n]
                # upgrade bool to int for augmented assignments
                if isinstance(obj, bool):
                    obj = int(-1)
                    self.tree.vardict[n] = obj
                node.obj = obj
        else:
            if n in ("__verilog__", "__vhdl__"):
                self.raiseError(node, _error.NotSupported,
                                "%s in generator function" % n)
            if n in self.globalRefs:
                self.raiseError(node, _error.UnboundLocal, n)
            self.refStack.add(n)

    def getName(self, node):
        n = node.id
        node.obj = None
        if n not in self.refStack:
            if (n in self.tree.vardict) and (n not in self.tree.nonlocaldict):
                self.raiseError(node, _error.UnboundLocal, n)
            self.globalRefs.add(n)
        if n in self.tree.sigdict:
            node.obj = sig = self.tree.sigdict[n]
            # mark shadow signal as driven only when they are seen somewhere
            if isinstance(sig, _ShadowSignal):
                sig._driven = 'wire'
            # mark tristate signal as driven if its driver is seen somewhere
            if isinstance(sig, _TristateDriver):
                sig._sig._driven = 'wire'
            if not isinstance(sig, _Signal):
                pass
            else:
                if sig._type is bool:
                    node.edge = sig.posedge
            if self.access == _access.INPUT:
                self.tree.inputs.add(n)
            elif self.access == _access.OUTPUT:
                self.tree.kind = _kind.TASK
                if n in self.tree.outputs:
                    node.kind = _kind.REG
                self.tree.outputs.add(n)
            elif self.access == _access.UNKNOWN:
                pass
            else:
                self.raiseError(node, _error.NotSupported, "Augmented signal assignment")
        if n in self.tree.vardict:
            obj = self.tree.vardict[n]
            if self.access == _access.INOUT: # probably dead code
                # upgrade bool to int for augmented assignments
                if isinstance(obj, bool):
                    obj = int(-1)
                    self.tree.vardict[n] = obj
            node.obj = obj
        elif n in self.tree.symdict:
            node.obj = self.tree.symdict[n]
            if _isTupleOfInts(node.obj):
                node.obj = _Rom(node.obj)
                self.tree.hasRom = True
            elif _isMem(node.obj):
                m = _getMemInfo(node.obj)
                if self.access == _access.INPUT:
                    m._read = True
                elif self.access == _access.OUTPUT:
                    m._driven = 'reg'
                    self.tree.outmems.add(n)
                elif self.access == _access.UNKNOWN:
                    pass
                else:
                    assert False, "unexpected mem access %s %s" % (n, self.access)
                self.tree.hasLos = True
            elif isinstance(node.obj, int):
                node.value = node.obj
            if n in self.tree.nonlocaldict:
                # hack: put nonlocal intbv's in the vardict
                self.tree.vardict[n] = v = node.obj
        elif n in builtins.__dict__:
            node.obj = builtins.__dict__[n]
        else:
            self.raiseError(node, _error.UnboundLocal, n)

    def visit_Return(self, node):
        self.raiseError(node, _error.NotSupported, "return statement")

    def visit_Print(self, node):
        self.tree.hasPrint = True
        f = []
        nr = 0
        a = []

        for n in node.args:
            if isinstance(n, ast.BinOp) and isinstance(n.op, ast.Mod) and \
               isinstance(n.left, ast.Constant) and isinstance(n.left.value, str):
                if isinstance(n.right, ast.Tuple):
                    a.extend(n.right.elts)
                else:
                    a.append(n.right)
                s = n.left.value
                while s:
                    if not s:
                        break
                    if s[:2] == "%%":
                        f.append("%")
                        s = s[2:]
                        continue
                    m = re_ConvSpec.match(s)
                    if m:
                        c = ConvSpec(**m.groupdict())
                        if c.justified != "RIGHT":
                            self.raiseError(node, _error.UnsupportedFormatString,
                                            "format justification specification: %s" % s)
                        if c.width != 0:
                            self.raiseError(node, _error.UnsupportedFormatString,
                                            "format width specification: %s" % s)
                        f.append(c)
                        s = s[m.end():]
                        nr += 1
                        continue
                    m = re_str.match(s)
                    if m:
                        f.append(s[:m.end()])
                        s = s[m.end():]
                        continue
                    self.raiseError(node, _error.UnsupportedFormatString, "%s" % s)
            elif isinstance(n, ast.Constant) and isinstance(n.value, str):
                f.append(n.value)
            else:
                f.append(defaultConvSpec)
                a.append(n)
                nr += 1
            f.append(" ")
        # remove last single space if it exists
        if f:
            f.pop()
        node.format = f
        node.args = a
        if len(node.args) < nr:
            self.raiseError(node, _error.FormatString, "not enough arguments")
        if len(node.args) > nr:
            self.raiseError(node, _error.FormatString, "too many arguments")
        self.generic_visit(node)

    def visit_Subscript(self, node):
        if isinstance(node.slice, ast.Slice):
            self.accessSlice(node)
        else:
            self.accessIndex(node)

    def accessSlice(self, node):
        self.visit(node.value)
        node.obj = self.getObj(node.value)
        self.access = _access.INPUT
        lower, upper = node.slice.lower, node.slice.upper
        if lower:
            self.visit(lower)
        if upper:
            self.visit(upper)
        if isinstance(node.obj, intbv):
            if self.kind == _kind.DECLARATION:
                self.require(lower, "Expected leftmost index")
                leftind = self.getVal(lower)
                if upper:
                    rightind = self.getVal(upper)
                else:
                    rightind = 0
                node.obj = node.obj[leftind:rightind]

    def accessIndex(self, node):
        self.visit(node.value)
        self.access = _access.INPUT
        if sys.version_info >= (3, 9, 0): # Python 3.9+: no ast.Index wrapper
            self.visit(node.slice)
        else:
            self.visit(node.slice.value)

        if isinstance(node.value.obj, _Ram):
            if isinstance(node.ctx, ast.Store):
                self.raiseError(node, _error.ListElementAssign)
            else:
                node.obj = node.value.obj.elObj
        elif _isMem(node.value.obj):
            node.obj = node.value.obj[0]
        elif isinstance(node.value.obj, _Rom):
            node.obj = int(-1)
        elif isinstance(node.value.obj, intbv):
            node.obj = bool()
        else:
            node.obj = bool()  # XXX default

    def visit_Tuple(self, node):
        self.generic_visit(node)

    def visit_While(self, node):
        node.breakLabel = _Label("BREAK")
        node.loopLabel = _Label("LOOP")
        self.labelStack.append(node.breakLabel)
        self.labelStack.append(node.loopLabel)
        self.visit(node.test)
        self.refStack.push()
        for n in node.body:
            self.visit(n)
        self.refStack.pop()
        y = node.body[0]
        if isinstance(y, ast.Expr):
            y = y.value
        if node.test.obj == True and \
           isinstance(y, ast.Yield) and \
           not self.tree.hasYield > 1 and \
           not isinstance(self.getObj(y.value), delay):
            node.kind = _kind.ALWAYS
            self.tree.senslist = y.senslist
        self.require(node, not node.orelse, "while-else not supported")
        self.labelStack.pop()
        self.labelStack.pop()

    def visit_Yield(self, node, *args):
        self.tree.hasYield += 1
        n = node.value
        self.visit(n)
        senslist = []
        if isinstance(n, ast.Tuple):
            for n in n.elts:
                if not isinstance(n.obj, (_Signal, _WaiterList)):
                    self.raiseError(node, _error.UnsupportedYield)
                senslist.append(n.obj)
        elif isinstance(n.obj, (_Signal, _WaiterList, delay)):
            senslist = [n.obj]
        elif _isMem(n.obj):
            senslist = n.obj
        else:
            self.raiseError(node, _error.UnsupportedYield)
        node.senslist = senslist


class _AnalyzeBlockVisitor(_AnalyzeVisitor):

    def __init__(self, tree):
        _AnalyzeVisitor.__init__(self, tree)
        for n, v in self.tree.symdict.items():
            if isinstance(v, _Signal):
                self.tree.sigdict[n] = v

    def visit_FunctionDef(self, node):
        self.refStack.push()
        for n in node.body:
            self.visit(n)
        self.tree.kind = _kind.ALWAYS
        for n in node.body[:-1]:
            if not self.getKind(n) == _kind.DECLARATION:
                self.tree.kind = _kind.INITIAL
                break
        if self.tree.kind == _kind.ALWAYS:
            w = node.body[-1]
            if not self.getKind(w) == _kind.ALWAYS:
                self.tree.kind = _kind.INITIAL
        self.refStack.pop()

    def visit_Module(self, node):
        self.generic_visit(node)
        for n in self.tree.outputs:
            s = self.tree.sigdict[n]
            if s._driven:
                self.raiseError(node, _error.SigMultipleDriven, n)
            s._driven = "reg"
        for n in self.tree.inputs:
            s = self.tree.sigdict[n]
            s._markRead()

    def visit_Return(self, node):
        # value should be None
        if node.value is None:
            pass
        elif isinstance(node.value, ast.Name) and node.value.id == "None":
            pass
        else:
            self.raiseError(node, _error.NotSupported, "return value other than None")


class _AnalyzeAlwaysCombVisitor(_AnalyzeBlockVisitor):

    def __init__(self, tree, senslist):
        _AnalyzeBlockVisitor.__init__(self, tree)
        self.tree.senslist = senslist

    def visit_FunctionDef(self, node):
        self.refStack.push()
        for n in node.body:
            self.visit(n)
        self.tree.kind = _kind.SIMPLE_ALWAYS_COMB
        for n in node.body:
            if isinstance(n, ast.Expr) and isinstance(n.value, ast.Str):
                continue  # skip doc strings
            if isinstance(n, ast.Assign) and \
               isinstance(n.targets[0], ast.Attribute) and \
               self.getKind(n.targets[0].value) != _kind.REG:
                pass
            else:
                self.tree.kind = _kind.ALWAYS_COMB
                return
        # rom access is expanded into a case statement in addition
        # to any always_comb that contains a list of signals
        # if self.tree.hasRom or self.tree.hasLos:
        if self.tree.hasRom:
            self.tree.kind = _kind.ALWAYS_COMB
        self.refStack.pop()

    def visit_Module(self, node):
        _AnalyzeBlockVisitor.visit_Module(self, node)
        if self.tree.kind == _kind.SIMPLE_ALWAYS_COMB:
            for n in self.tree.outputs:
                s = self.tree.sigdict[n]
                s._driven = "wire"
            for n in self.tree.outmems:
                m = _getMemInfo(self.tree.symdict[n])
                m._driven = "wire"


class _AnalyzeAlwaysSeqVisitor(_AnalyzeBlockVisitor):

    def __init__(self, tree, senslist, reset, sigregs, varregs):
        _AnalyzeBlockVisitor.__init__(self, tree)
        self.tree.senslist = senslist
        self.tree.reset = reset
        self.tree.sigregs = sigregs
        self.tree.varregs = varregs

    def visit_FunctionDef(self, node):
        self.refStack.push()
        for n in node.body:
            self.visit(n)
        self.tree.kind = _kind.ALWAYS_SEQ
        self.refStack.pop()


class _AnalyzeAlwaysDecoVisitor(_AnalyzeBlockVisitor):

    def __init__(self, tree, senslist):
        _AnalyzeBlockVisitor.__init__(self, tree)
        self.tree.senslist = senslist
        for arg in senslist:
            if isinstance(arg, delay):
                self.raiseError(_error.NotSupported, "delay argument in @always decorator")

    def visit_FunctionDef(self, node):
        self.refStack.push()
        for n in node.body:
            self.visit(n)
        self.tree.kind = _kind.ALWAYS_DECO
        self.refStack.pop()


class _AnalyzeFuncVisitor(_AnalyzeVisitor):

    def __init__(self, tree, args, keywords):
        _AnalyzeVisitor.__init__(self, tree)
        self.args = args
        self.keywords = keywords
        self.tree.hasReturn = False
        self.tree.returnObj = None

    def visit_FunctionDef(self, node):
        self.refStack.push()
        argnames = _get_argnames(node)
        for i, arg in enumerate(self.args):
            n = argnames[i]
            self.tree.symdict[n] = self.getObj(arg)
            self.tree.argnames.append(n)
        for kw in self.keywords:
            n = kw.arg
            self.tree.symdict[n] = self.getObj(kw.value)
            self.tree.argnames.append(n)
        for n, v in self.tree.symdict.items():
            if isinstance(v, (_Signal, intbv)):
                self.tree.sigdict[n] = v
        for stmt in node.body:
            self.visit(stmt)
        self.refStack.pop()
        if self.tree.hasYield:
            self.raiseError(node, _error.NotSupported,
                            "call to a generator function")
        if self.tree.kind == _kind.TASK:
            if self.tree.returnObj is not None:
                self.raiseError(node, _error.NotSupported,
                                "function with side effects and return value")
        else:
            if self.tree.returnObj is None:
                self.raiseError(node, _error.NotSupported,
                                "pure function without return value")

    def visit_Return(self, node):
        self.kind = _kind.DECLARATION
        if node.value is not None:
            self.visit(node.value)
        self.kind = _kind.NORMAL
        if node.value is None:
            obj = None
        elif isinstance(node.value, ast.Name) and node.value.id == 'None':
            obj = None
        elif node.value.obj is not None:
            obj = node.value.obj
        else:
            self.raiseError(node, _error.ReturnTypeInfer)
        if isinstance(obj, intbv) and len(obj) == 0:
            self.raiseError(node, _error.ReturnIntbvBitWidth)
        if self.tree.hasReturn:
            returnObj = self.tree.returnObj
            if isinstance(obj, type(returnObj)):
                pass
            elif isinstance(returnObj, type(obj)):
                self.tree.returnObj = obj
            else:
                self.raiseError(node, _error.ReturnTypeMismatch)
            if getNrBits(obj) != getNrBits(returnObj):
                self.raiseError(node, _error.ReturnNrBitsMismatch)
        else:
            self.tree.returnObj = obj
            self.tree.hasReturn = True


# a local function to drill down to the last interface
def expandinterface(v, name, obj):
    for attr, attrobj in vars(obj).items():
        if isinstance(attrobj, _Signal):
# override any 'mangled' name
#             signame = attrobj._name
#             if not signame:
            signame = name + '_' + attr
            attrobj._name = signame
            v.argdict[signame] = attrobj
            v.argnames.append(signame)
        elif isinstance(attrobj, myhdl.EnumType):
            pass
        elif hasattr(attrobj, '__dict__'):
            # can assume is yet another interface ...
            expandinterface(v, name + '_' + attr, attrobj)


def _analyzeTopFunc(func, *args, **kwargs):
    tree = _makeAST(func)
    v = _AnalyzeTopFuncVisitor(func, tree, *args, **kwargs)
    v.visit(tree)

    objs = []
    for name, obj in v.fullargdict.items():
        if not isinstance(obj, _Signal):
            objs.append((name, obj))

    # create ports for any signal in the top instance if it was buried in an
    # object passed as in argument

    # now expand the interface objects
    for name, obj in objs:
        if hasattr(obj, '__dict__'):
            # must be an interface object (probably ...?)
            expandinterface(v, name, obj)

    return v


class _AnalyzeTopFuncVisitor(_AnalyzeVisitor):

    def __init__(self, func, tree, *args, **kwargs):
        self.func = func
        self.tree = tree
        self.args = args
        self.kwargs = kwargs
        self.name = None
        self.fullargdict = {}
        self.argdict = {}
        self.argnames = []

    def visit_FunctionDef(self, node):
        self.name = node.name
        if isboundmethod(self.func):
            if isinstance(self.func.__self__, HdlClass):
                # must find names ...
                for arg in self.args:
                    # be selective
                    if isinstance(arg, _Signal):
                        self.argnames.append(arg._name)
                    elif _isListOfSigs(arg):
                        raise NotImplementedError(f'do not handle ListOfSignals {self.name}:{arg}')
            else:
                # another class
                self.argnames = _get_argnames(node)
                if not self.argnames[0] == 'self':
                    self.raiseError(node, _error.NotSupported,
                                    "first method argument name other than 'self'")
                # skip self
                self.argnames = self.argnames[1:]
        else:
            self.argnames = _get_argnames(node)

        i = -1
        for i, arg in enumerate(self.args):
            n = self.argnames[i]
            self.fullargdict[n] = arg
            if isinstance(arg, _Signal):
                self.argdict[n] = arg
            if _isMem(arg):
                self.raiseError(node, _error.ListAsPort, n)
        for n in self.argnames[i + 1:]:
            if n in self.kwargs:
                arg = self.kwargs[n]
                self.fullargdict[n] = arg
                if isinstance(arg, _Signal):
                    self.argdict[n] = arg
                if _isMem(arg):
                    self.raiseError(node, _error.ListAsPort, n)
        self.argnames = [n for n in self.argnames if n in self.argdict]



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for traceSignals """
import os
import random

import pytest

from myhdl import block, Signal, Simulation, _simulator, delay, instance, intbv
from myhdl._traceSignals import TraceSignalsError, _error, traceSignals
from helpers import raises_kind

random.seed(1)  # random, but deterministic
path = os.path

QUIET = 1


@block
def gen(clk):

    @instance
    def comb():
        while 1:
            yield delay(10)
            clk.next = not clk

    return comb


@block
def fun():
    clk = Signal(bool(0))
    inst = gen(clk)
    return inst


@block
def dummy():
    clk = Signal(bool(0))
    inst = gen(clk)  # inst will not be returned ...
    return 1


@block
def top():
    inst = traceSignals(fun())
    return inst


@block
def top2():
    inst = [{} for __ in range(4)]
    j = 3
    inst[j - 2]['key'] = traceSignals(fun())
    return inst


@block
def top3():
    inst_1 = traceSignals(fun())
    inst_2 = traceSignals(fun())
    return inst_1, inst_2


@block
def genTristate(clk, x, y, z):
    xd = x.driver()
    yd = y.driver()
    zd = z.driver()

    @instance
    def ckgen():
        while 1:
            yield delay(10)
            clk.next = not clk

    @instance
    def comb():
        for v in [True, False, None, 0, True, None, None, 1]:
            yield clk.posedge
            xd.next = v
            if v is None:
                yd.next = zd.next = None
            elif v:
                yd.next = zd.next = 11
            else:
                yd.next = zd.next = 0

    return ckgen, comb


@block
def tristate():
    from myhdl import TristateSignal
    clk = Signal(bool(0))
    x = TristateSignal(True)  # single bit
    y = TristateSignal(intbv(0))  # intbv with undefined width
    z = TristateSignal(intbv(0)[8:])  # intbv with fixed width

    inst = genTristate(clk, x, y, z)
    return inst


@block
def topTristate():
    inst = traceSignals(tristate())
    return inst


@pytest.fixture
def vcd_dir(tmpdir):
    with tmpdir.as_cwd():
        yield tmpdir
    if _simulator._tracing:
        _simulator._tf.close()
        _simulator._tracing = 0


class TestTraceSigs:

    # TODO: multiple trace handling is different now has the
    # calls go bottom-up. To be revisited.
    # def testMultipleTraces(self, vcd_dir):
    #     with raises_kind(TraceSignalsError, _error.MultipleTraces):
    #         dut = top3()

    def testArgType1(self, vcd_dir):
        with raises_kind(TraceSignalsError, _error.ArgType):
            dut = traceSignals([1, 2])  # dut will not be returned ...

    # this test is no longer relevant
    # def testReturnVal(self, vcd_dir):
    #     from myhdl import ExtractHierarchyError
    #     from myhdl._extractHierarchy import _error
    #     kind = _error.InconsistentToplevel % (2, "dummy")
    #     with raises_kind(ExtractHierarchyError, kind):
    #         dut = traceSignals(dummy())

    def testHierarchicalTrace1(self, vcd_dir):
        p = "%s.vcd" % fun.__name__
        top()
        assert path.exists(p)

    def testHierarchicalTrace2(self, vcd_dir):
        pdut = "%s.vcd" % top.__name__
        psub = "%s.vcd" % fun.__name__
        dut = traceSignals(top())  # dut will not be returned ...
        assert path.exists(pdut)
        assert not path.exists(psub)

    def testTristateTrace(self, vcd_dir):
        sim = Simulation(topTristate())
        sim.run(100, quiet=QUIET)
        sim.quit()

    def testBackupOutputFile(self, vcd_dir):
        p = "%s.vcd" % fun.__name__

        dut = traceSignals(fun())
        sim = Simulation(dut)
        sim.run(1000, quiet=QUIET)
        sim.quit()

        _simulator._tf.close()
        _simulator._tracing = 0
        size = path.getsize(p)
        pbak = p[:-4] + '.' + str(path.getmtime(p)) + '.vcd'
        assert not path.exists(pbak)
        dut = traceSignals(fun())
        _simulator._tf.close()
        _simulator._tracing = 0
        assert path.exists(p)
        assert path.exists(pbak)
        assert path.getsize(pbak) == size
        assert path.getsize(p) < size

    def testSetDirectory(self, vcd_dir):
        traceSignals.directory = 'some_vcd_dir'
        os.mkdir(path.join(str(vcd_dir), traceSignals.directory))
        pdut = "%s.vcd" % top.__name__
        psub = "%s.vcd" % fun.__name__
        pdutd = path.join(traceSignals.directory, "%s.vcd" % top.__name__)
        psubd = path.join(traceSignals.directory, "%s.vcd" % fun.__name__)
        dut = traceSignals(top())  # dut will not be returned ...
        _simulator._tf.close()
        _simulator._tracing = 0
        traceSignals.directory = None
        assert not path.exists(pdut)
        assert not path.exists(psub)
        assert path.exists(pdutd)
        assert not path.exists(psubd)



import myhdl
from myhdl import *

from rs232_util import sec, parity, ParityError, StopBitError


def rs232_rx(rx, actual, cfg):

    """ rs232 receiver.

    rx -- serial input data
    actual -- data actually received
    cfg -- rs232_util.Config configuration object

    """

    @instance
    def comb():
        data = intbv(0)
        period = int(1 * sec / cfg.baud_rate)

        yield rx.posedge
        yield delay(period // 2)

        data[7] = 0
        for i in downrange(cfg.n_bits):
            yield delay(period)
            data[i] = rx

        if cfg.parity is not None:
            yield delay(period)
            if rx != parity(data, cfg.parity):
                raise ParityError

        yield delay(period)
        if rx != 0:
            raise StopBitError

        actual[8:] = data

    return comb




from myhdl import (block, Signal, instances, always_comb)

from myhdl import ConversionError
from myhdl.conversion._misc import _error


@block
def SubFunction_1837003(xout, yout, xin, yin):

    @always_comb
    def comb():
        x = 4
        y = 2
        xout.next = xin
        yout.next = yin

    return instances()

# def Function_1837003(xout,yout,x,y):
#    return SubFunction_1837003(xout,yout,x,y)


x = Signal(bool(0))
y = Signal(bool(0))
xout = Signal(bool(0))
yout = Signal(bool(0))
xin = Signal(bool(0))
yin = Signal(bool(0))


def test_bug_1837003():
    try:
        SubFunction_1837003(xout, yout, x, y).convert(hdl='Verilog')
    except ConversionError as e:
        assert e.kind == _error.ShadowingVar
    else:
        assert False



import myhdl
from myhdl import *

from timer import timer_sig, timer_var

def test_timer(timer):

    MAXVAL = 1234

    clock = Signal(bool())
    reset = Signal(bool())
    flag = Signal(bool())

    dut = timer(flag, clock, reset, MAXVAL)

    @instance
    def clkgen():
        clock.next = 0
        reset.next = 0
        yield delay(10)
        reset.next = 1
        yield delay(10)
        reset.next = 0
        yield delay(10)
        for i in range(2**25):
            clock.next = not clock
            yield delay(10)

    @instance
    def monitor():
        count = intbv(0, min=0, max=MAXVAL+1)
        seen = False
        while True:
            yield clock.posedge
            if seen:
                if flag:
                    assert count == MAXVAL
                else:
                    count += 1
            if flag:
                seen = True
                count[:] = 0

    return dut, clkgen, monitor

if __name__ == '__main__':
    sim = Simulation(test_timer(timer_var))
    sim.run()
        



import myhdl
from myhdl import *


def random_generator(random_word, enable, clock, reset):

    W = len(random_word)

    @instance
    def comb():
        lfsr = modbv(0)[64:]
        word = modbv(0)[W:]
        while True:
            yield clock.posedge, reset.posedge
            if reset == 1:
                random_word.next = 0
                lfsr[:] = 1
            else:
                if enable:
                    for i in range(W):
                        word[i] = lfsr[63]
                        tmp0 = lfsr[63] ^ lfsr[62] ^ lfsr[60] ^ lfsr[59]
                        lfsr <<= 1
                        lfsr[0] = tmp0
                    random_word.next = word

    return comb

# def random_generator(random_word, enable, clock, reset):

#     W = len(random_word)

#     @instance
#     def comb():
#         lfsr = intbv(0)[64:]
#         word =  intbv(0)[W:]
#         while True:
#             yield clock.posedge, reset.posedge
#             if reset == 1:
#                 random_word.next = 0
#                 lfsr[:] = 0
#                 lfsr[63] = 1
#             else:
#                 if enable:
#                     for i in range(W):
#                         word[i] = lfsr[0]
#                         tmp63 = lfsr[0] ^ lfsr[1] ^ lfsr[3] ^ lfsr[4]
#                         lfsr >>= 1
#                         lfsr[63] = tmp63
#                     random_word.next = word

#     return comb




from myhdl import block, always_seq, Signal, intbv, enum

ACTIVE_LOW = 0
FRAME_SIZE = 8
t_state = enum('SEARCH', 'CONFIRM', 'SYNC')

@block
def framer_ctrl(sof, state, sync_flag, clk, reset_n):

    """ Framing control FSM.

    sof -- start-of-frame output bit
    state -- FramerState output
    sync_flag -- sync pattern found indication input
    clk -- clock input
    reset_n -- active low reset

    """

    index = Signal(intbv(0, min=0, max=FRAME_SIZE)) # position in frame

    @always_seq(clk.posedge, reset=reset_n)
    def FSM():
        if reset_n == ACTIVE_LOW:
            sof.next = 0
            index.next = 0
            state.next = t_state.SEARCH

        else:
            index.next = (index + 1) % FRAME_SIZE
            sof.next = 0

            if state == t_state.SEARCH:
                index.next = 1
                if sync_flag:
                    state.next = t_state.CONFIRM

            elif state == t_state.CONFIRM:
                if index == 0:
                    if sync_flag:
                        state.next = t_state.SYNC
                    else:
                        state.next = t_state.SEARCH

            elif state == t_state.SYNC:
                if index == 0:
                    if not sync_flag:
                        state.next = t_state.SEARCH
                sof.next = (index == FRAME_SIZE-1)

            else:
                raise ValueError("Undefined state")

    return FSM



import os
path = os.path
import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, intbv, delay, always, instance, StopSimulation)

ACTIVE_LOW, INACTIVE_HIGH = 0, 1


@block
def decRef(count, enable, clock, reset, n):
    """ Decrementer with enable.

    count -- output
    enable -- control input, decrement when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    """

    @instance
    def comb():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    if count == -n:
                        count.next = n - 1
                    else:
                        count.next = count - 1

    return comb


@block
def dec(count, enable, clock, reset, n):
    """ Decrementer with enable.

    count -- output
    enable -- control input, decrement when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    """

    @instance
    def decProcess():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable == True:
                    if count == -n:
                        count.next = n - 1
                    else:
                        count.next = count - 1

    return decProcess


@block
def decFunc(count, enable, clock, reset, n):

    def decFuncFunc(cnt):
        count_next = intbv(0, min=-n, max=n)
        if cnt == -n:
            count_next[:] = n - 1
        else:
            count_next[:] = cnt - 1
        return count_next

    @always(clock.posedge, reset.negedge)
    def decFuncGen():
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if enable:
                count.next = decFuncFunc(count)

    return decFuncGen


@block
def decTask(count, enable, clock, reset, n):

    def decTaskFunc(cnt, enable, n):
        if enable:
            if cnt == -n:
                cnt[:] = n - 1
            else:
                cnt[:] = cnt - 1

    @instance
    def decTaskGen():
        cnt = intbv(0, min=-n, max=n)
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                cnt[:] = 0
                count.next = 0
            else:
                # print count
                decTaskFunc(cnt, enable, n)
                count.next = cnt

    return decTaskGen


@block
def decTaskFreeVar(count, enable, clock, reset, n):

    def decTaskFunc():
        if enable:
            if count == -n:
                count.next = n - 1
            else:
                count.next = count - 1

    @instance
    def decTaskGen():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                # print count
                decTaskFunc()

    return decTaskGen


@block
def DecBench(dec):

    m = 8
    n = 2 ** (m - 1)

    count = Signal(intbv(0, min=-n, max=n))
    enable = Signal(bool(0))
    clock, reset = [Signal(bool(1)) for __ in range(2)]

    @instance
    def clockGen():
        yield delay(10)
        clock.next = 0
        while 1:
            yield delay(10)
            clock.next = not clock

    enables = tuple([min(1, randrange(5)) for i in range(1000)])

    @instance
    def stimulus():
        reset.next = INACTIVE_HIGH
        yield clock.negedge
        reset.next = ACTIVE_LOW
        yield clock.negedge
        reset.next = INACTIVE_HIGH
        for i in range(1000):
            enable.next = 1
            yield clock.negedge
        for i in range(len(enables)):
            enable.next = enables[i]
            yield clock.negedge
        raise StopSimulation

    @instance
    def check():
        yield reset.negedge
        yield reset.posedge
        print(count)
        while 1:
            yield clock.posedge
            yield delay(1)
            print(count)

    dec_inst = dec(count, enable, clock, reset, n=n)

    return dec_inst, clockGen, stimulus, check


def testDecRef():
    assert DecBench(decRef).verify_convert() == 0


def testDec():
    assert DecBench(dec).verify_convert() == 0


def testDecFunc():
    assert DecBench(decFunc).verify_convert() == 0


def testDecTask():
    assert DecBench(decTask).verify_convert() == 0

# # def testDecTaskFreeVar():
# #     assert verify(DecBench, decTaskFreeVar) == 0

# #     def testDecRef(self):
# #         sim = self.bench(decRef)
# #         sim.run(quiet=1)

# #     def testDec(self):
# #         sim = self.bench(dec)
# #         sim.run(quiet=1)

# #     def testDecFunc(self):
# #         sim = self.bench(decFunc)
# #         sim.run(quiet=1)

# # signed inout in task doesn't work yet in Icarus
# #     def testDecTask(self):
# #         sim = self.bench(decTask)
# #         sim.run(quiet=1)

# #     def testDecTaskFreeVar(self):
# #         sim = self.bench(decTaskFreeVar)
# #         sim.run(quiet=1)



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run cosimulation unit tests. """


import sys

sys.path.append("../../test")
sys.path.append("../../../example/manual")

import test_gray_properties, test_gray_original, test_inc, test_dff

modules = (test_gray_properties, test_gray_original, test_inc, test_dff )

import unittest

tl = unittest.defaultTestLoader
def suite():
    alltests = unittest.TestSuite()
    for m in modules:
        alltests.addTest(tl.loadTestsFromModule(m))
    return alltests

def main():
    unittest.main(defaultTest='suite',
                  testRunner=unittest.TextTestRunner(verbosity=2))
    

if __name__ == '__main__':
    main()



from myhdl import (block, Signal, intbv, delay, always_comb,
                   always, instance, StopSimulation)
from myhdl._Simulation import Simulation
from myhdl.conversion import verify


class HdlObj(object):

    def __init__(self):
        pass

    @block
    def method_func(self, clk, srst, x, y):
        z = Signal(intbv(0, min=y.min, max=y.max))
        ifx = self._mfunc(x, z)

        @always(clk.posedge)
        def hdl():
            if srst:
                y.next = 0
            else:
                y.next = x + z  # x + (x+1)

        return hdl, ifx

    @block
    def _mfunc(self, x, y):

        @always_comb
        def _hdl():
            y.next = x + 1

        return _hdl


@block
def _func(x, y):

    @always_comb
    def _hdl():
        y.next = x + 1

    return _hdl


class HdlObjObj(object):

    def __init__(self):
        pass

    @block
    def method_func(self, clk, srst, x, y):
        z1 = Signal(intbv(0, min=y.min, max=y.max))
        z2 = Signal(intbv(0, min=y.min, max=y.max))
        hobj = HdlObj()
        ifx1 = hobj._mfunc(x, z1)
        ifx2 = _func(x, z2)

        @always(clk.posedge)
        def hdl():
            if srst:
                y.next = 0
            else:
                y.next = x + z1 + (z1 - z2)

        return hdl, ifx1, ifx2


class HdlObjAttrSimple(object):

    def __init__(self):
        self.AConstant = 3

    @block
    def method_func(self, clk, srst, x, y):

        # limitation for class method conversion, the object attributes
        # can only be used/accessed during elaboration.
        AConstant = int(self.AConstant)

        @always(clk.posedge)
        def hdl():
            if srst:
                y.next = 0
            else:
                y.next = x + (x + 1) + AConstant - 3

        return hdl


class HdlObjAttr(object):

    def __init__(self, clk, srst, x, y):
        self.clk = clk
        self.srst = srst
        self.x = x
        self.y = y
        self.z = Signal(intbv(0, min=y.min, max=y.max))
        self.hobj = HdlObj()

    @block
    def method_func(self):
        ifx = self.hobj._mfunc(self.x, self.z)

        @always(self.clk.posedge)
        def hdl():
            if self.srst:
                self.y.next = 0
            else:
                self.y.next = self.x + self.z

        return hdl, ifx


@block
def ObjBench(hObj):

    clk = Signal(False)
    srst = Signal(False)
    x = Signal(intbv(0, min=0, max=16))
    y = Signal(intbv(0, min=0, max=16))

    if hObj == HdlObjAttr:
        hdlobj_inst = hObj(clk, srst, x, y)
        hdl_inst = hdlobj_inst.method_func()
    elif hObj == HdlObjAttrSimple:
        hdlobj_inst = hObj()
        hdl_inst = hdlobj_inst.method_func(clk, srst, x, y)
    elif hObj == HdlObj or hObj == HdlObjObj:
        hdlobj_inst = hObj()
        hdl_inst = hdlobj_inst.method_func(clk, srst, x, y)
    else:
        raise ValueError("Incorrect hOjb %s" % (type(hObj), str(hObj)))

    @instance
    def tb_clkgen():
        clk.next = False
        srst.next = False
        yield delay(10)
        srst.next = True
        yield delay(10)
        srst.next = False
        yield delay(10)
        for dummy in range(1000):
            yield delay(10)
            clk.next = not clk

    xtable = (1, 2, 3, 4, 5, 6)
    ytable = (3, 5, 7, 9, 11, 13)

    @instance
    def tb_stimulus():
        for ii in range(30):
            yield clk.posedge
        assert len(xtable) == len(ytable)
        for ii in range(len(xtable)):
            nx = xtable[ii]
            ny = ytable[ii]
            x.next = nx
            yield clk.posedge
            yield clk.posedge
            print("x %d y %d" % (x, y))
            assert x == nx
            assert y == ny
        raise StopSimulation

    return hdl_inst, tb_clkgen, tb_stimulus


def test_hdlobj():
    assert verify(ObjBench(HdlObj)) == 0


def test_hdlobjobj():
    assert verify(ObjBench(HdlObjObj)) == 0


def test_hdlobjattrsimple():
    assert verify(ObjBench(HdlObjAttrSimple)) == 0

# def test_hdlobjattr():
#    # object attributes currently not supported, these
#    # tests are for class method conversion only and not
#    # class attribute conversion.  When (if) class attribute
#    # is supported remove this test.
#    assert verify(ObjBench, HdlObjAttr) == 1


if __name__ == '__main__':
    Simulation(ObjBench(HdlObj)).run()
    Simulation(ObjBench(HdlObjAttrSimple)).run()
    Simulation(ObjBench(HdlObjAttr)).run()



import py
import pytest

from myhdl.conversion import analyze, verify
from myhdl.conversion._verify import _simulators

xfail = pytest.mark.xfail

all_sims = list(_simulators)

collect_ignore = ['conversion/toVerilog/test_not_supported_py2.py']

def pytest_addoption(parser):
    parser.addoption("--sim", action="store", choices=all_sims,
                     help="HDL Simulator")


def pytest_configure(config):
    sim = config.getoption('sim')
    if sim is not None:
        verify.simulator = analyze.simulator = sim


def pytest_report_header(config):
    sim = config.getoption('sim')
    if config.getoption('sim') is not None:
        hdr = ['Simulator: {sim}']
        if not py.path.local.sysfind(sim):
            hdr += ['Warning: {sim} not found in PATH']
        return '\n'.join(hdr).format(sim=sim)


def bug(issue_no, hdl='all'):
    if hdl == 'all':
        sims = all_sims
    else:
        sims = [k for k, v in _simulators.items() if v.hdl.lower() == hdl]
    return xfail(verify.simulator in sims, reason='issue '+issue_no)



import os
path = os.path
import unittest
from random import randrange

from myhdl import (block, Signal, intbv, delay,
                   instance, concat, downrange)
from myhdl._Simulation import Simulation
from .util import setupCosimulation

COSET = 0x55


def calculateHecRef(header):
    """ Return hec for an ATM header.

    Reference version.
    The hec polynomial is 1 + x + x**2 + x**8.
    """
    hec = intbv(0)
    for bit in header[32:]:
        hec[8:] = concat(hec[7:2],
                         bit ^ hec[1] ^ hec[7],
                         bit ^ hec[0] ^ hec[7],
                         bit ^ hec[7]
                        )
    return hec ^ COSET


def calculateHecFunc(header):
    """ Return hec for an ATM header.
    
    Translatable version.
    The hec polynomial is 1 + x + x**2 + x**8.
    """
    h = intbv(0)[8:]
    for i in downrange(len(header)):
        bit = header[i]
        h[:] = concat(h[7:2],
                      bit ^ h[1] ^ h[7],
                      bit ^ h[0] ^ h[7],
                      bit ^ h[7]
                      )
    h ^= COSET
    return h


def calculateHecTask(hec, header):
    """ Calculate hec for an ATM header.
    
    Translatable version.
    The hec polynomial is 1 + x + x**2 + x**8.
    """
    h = intbv(0)[8:]
    for i in downrange(len(header)):
        bit = header[i]
        h[:] = concat(h[7:2],
                      bit ^ h[1] ^ h[7],
                      bit ^ h[0] ^ h[7],
                      bit ^ h[7]
                      )
    h ^= COSET
    hec[:] = h


@block
def HecCalculatorPlain(hec, header):
    """ Hec calculation module.

    Plain version.
    """

    @instance
    def comb():
        h = intbv(0)[8:]
        while 1:
            yield header
            h[:] = 0
            for i in downrange(len(header)):
                bit = header[i]
                h[:] = concat(h[7:2],
                              bit ^ h[1] ^ h[7],
                              bit ^ h[0] ^ h[7],
                              bit ^ h[7]
                              )
            hec.next = h ^ COSET

    return comb


@block
def HecCalculatorFunc(hec, header=1):
    """ Hec calculation module.

    Version with function call.
    """

    @instance
    def comb():
        while 1:
            yield header
            hec.next = calculateHecFunc(header=header)

    return comb


@block
def HecCalculatorTask(hec, header):
    """ Hec calculation module.

    Version with task call.
    """

    @instance
    def comb():
        h = intbv(0)[8:]
        while 1:
            yield header
            calculateHecTask(h, header)
            hec.next = h

    return comb


@block
def HecCalculatorTask2(hec, header):
    """ Hec calculation module.

    Version with task call.
    """

    @instance
    def comb():
        h = intbv(0)[8:]
        while 1:
            yield header
            calculateHecTask(header=header, hec=h)
            hec.next = h

    return comb


@block
def HecCalculator_v(name, hec, header):
    return setupCosimulation(**locals())


headers = [ 0x00000000,
            0x01234567,
            0xbac6f4ca
          ]

headers.extend([randrange(2 ** 32 - 1) for i in range(10)])


class TestHec(unittest.TestCase):

    def bench(self, HecCalculator):

        hec = Signal(intbv(0)[8:])
        hec_v = Signal(intbv(0)[8:])
        header = Signal(intbv(-1)[32:])

        heccalc_inst = HecCalculator(hec, header).convert(hdl='Verilog')
        # heccalc_inst = HecCalculator(hec, header)
        heccalc_v_inst = HecCalculator_v(HecCalculator.__name__, hec_v, header)

        def stimulus():
            for h in headers:
                header.next = h
                yield delay(10)
                hec_ref = calculateHecRef(header)
                # print "hec: %s hec_v: %s" % (hex(hec), hex(hec_v))
                self.assertEqual(hec, hec_ref)
                self.assertEqual(hec, hec_v)

        return stimulus(), heccalc_inst, heccalc_v_inst

    def testPlain(self):
        sim = self.bench(HecCalculatorPlain)
        Simulation(sim).run()

    def testFunc(self):
        sim = self.bench(HecCalculatorFunc)
        Simulation(sim).run()

    def testTask(self):
        sim = self.bench(HecCalculatorTask)
        Simulation(sim).run()

    def testTask2(self):
        sim = self.bench(HecCalculatorTask2)
        Simulation(sim).run()


if __name__ == '__main__':
    unittest.main()



from myhdl import (block, Signal, enum, intbv, always_comb, instances)

from .util import verilogCompile

#############################
# bug report (Tom Dillon)
# conflicts in reg/wire names
#############################

width = 8


@block
def add(x, a, b):

    def comb():
        x.next = a + b

    L0 = always_comb(comb)
    return L0


@block
def add3(x, a, b, c):
    x0 = Signal(intbv(0, min=-2 ** (width - 1), max=2 ** (width - 1)))
    A0 = add(x0, a, b)
    A1 = add(x, x0, c)

    return instances()


@block
def DUT_Verilog_001(x, a, b, c, d, e):
    x0 = Signal(intbv(0, min=-2 ** (width - 1), max=2 ** (width - 1)))

    A0 = add3(x0, a, b, c)
    A1 = add3(x, x0, d, e)

    return instances()


def test_bugreports_001():
    x, a, b, c, d, e = [Signal(intbv(0, min=-2 ** (width - 1), max=2 ** (width - 1))) for i in range(6)]

    DUT_Verilog_001(x, a, b, c, d, e).convert(hdl='Verilog')
    # verilogCompile(TestModule.__name__)

# test()

##############################
# Bug report (Tom Dillon)
# Conflicts in reg/wire names
###############################

# @block
# def add(x, a, b):
#
#     def comb():
#         x.next = a + b
#
#     L0 = always_comb(comb)
#     return L0


@block
def add4(x, a, b, c, d):
    xL = [Signal(intbv(0, min=-2 ** (width + 2), max=2 ** (width + 2))) for i in range(2)]

    # xl0 = Signal(intbv(0,min=-2**(width+2),max=2**(width+2)))
    # xl1 = Signal(intbv(0,min=-2**(width+2),max=2**(width+2)))

    A0 = add(xL[0], a, b)
    A1 = add(xL[1], xL[0], c)
    A2 = add(x, xL[1], d)

    return instances()


@block
def DUT_Verilog_002(x, a, b, c, d, e):
    x0 = Signal(intbv(0, min=-2 ** (width + 2), max=2 ** (width + 2)))

    A0 = add4(x0, a, b, c, d)
    A1 = add4(x, x0, e, a, b)

    return instances()


def test_bugreports_002():
    width = 8

    x, a, b, c, d, e = [Signal(intbv(0, min=-2 ** (width - 1), max=2 ** (width - 1))) for __ in range(6)]

    DUT_Verilog_002(x, a, b, c, d, e).convert(hdl='Verilog')
    verilogCompile(DUT_Verilog_002.__name__)

# test()

###################################
# Bug report (George Pantazopoulos)
# case variable name in embedded FSM
####################################


from .test_fsm import FramerCtrl


@block
def mid(SOF, clk, reset_n):
    t_State = enum('SEARCH', 'CONFIRM', 'SYNC')
    syncFlag = Signal(bool(0))
    state = Signal(t_State.SEARCH)

    fsm_1 = FramerCtrl(SOF, state, syncFlag, clk, reset_n, t_State)

    return fsm_1


@block
def DUT_Verilog_003(SOF, clk, reset_n):
    mid_1 = mid(SOF, clk, reset_n)
    return mid_1


def test_bugreports_003():
    clk = Signal(bool(0))
    reset_n = Signal(bool(1))
    SOF = Signal(bool(0))

    DUT_Verilog_003(SOF, clk, reset_n).convert()
    verilogCompile(DUT_Verilog_003.__name__)

# test()




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the intbv unit tests. """
import operator
import random
import sys
from copy import copy, deepcopy
from random import randrange

import pytest

from myhdl import intbv

random.seed(2)  # random, but deterministic
maxint = sys.maxsize


class TestIntbvInit:

    def testDefaultValue(self):
        assert intbv() == 0


def getItem(s, i):
    ext = '0' * (i - len(s) + 1)
    exts = ext + s
    si = len(exts) - 1 - i
    return exts[si]


def getSlice(s, i, j):
    ext = '0' * (i - len(s) + 1)
    exts = ext + s
    si = len(exts) - i
    sj = len(exts) - j
    return exts[si:sj]


def getSliceLeftOpen(s, j):
    ext = '0' * (j - len(s) + 1)
    exts = ext + s
    if j:
        return exts[:-j]
    else:
        return exts


def setItem(s, i, val):
    ext = '0' * (i - len(s) + 1)
    exts = ext + s
    si = len(exts) - 1 - i
    return exts[:si] + val + exts[si + 1:]


def setSlice(s, i, j, val):
    ext = '0' * (i - len(s) + 1)
    exts = ext + s
    si = len(exts) - i
    sj = len(exts) - j
    return exts[:si] + val[si - sj:] + exts[sj:]


def setSliceLeftOpen(s, j, val):
    ext = '0' * (j - len(s) + 1)
    exts = ext + s
    if j:
        return val + exts[-j:]
    else:
        return val


class TestIntBvIndexing:

    def seqsSetup(self):
        seqs = ["0", "1", "000", "111", "010001", "110010010", "011010001110010"]
        seqs.extend(["0101010101", "1010101010", "00000000000", "11111111111111"])
        seqs.append("11100101001001101000101011011101001101")
        seqs.append("00101011101001011111010100010100100101010001001")
        self.seqs = seqs
        seqv = ["0", "1", "10", "101", "1111", "1010"]
        seqv.extend(["11001", "00111010", "100111100"])
        seqv.append("0110101001111010101110011010011")
        seqv.append("1101101010101101010101011001101101001100110011")
        self.seqv = seqv

    def testGetItem(self):
        self.seqsSetup()
        for s in self.seqs:
            n = int(s, 2)
            bv = intbv(n)
            bvi = intbv(~n)
            for i in range(len(s) + 20):
                ref = int(getItem(s, i), 2)
                res = bv[i]
                resi = bvi[i]
                assert res == ref
                assert type(res) == bool
                assert resi == ref ^ 1
                assert type(resi) == bool

    def testGetSlice(self):
        self.seqsSetup()
        for s in self.seqs:
            n = int(s, 2)
            bv = intbv(n)
            bvi = intbv(~n)
            for i in range(1, len(s) + 20):
                for j in range(0, len(s) + 20):
                    try:
                        res = bv[i:j]
                        resi = bvi[i:j]
                    except ValueError:
                        assert i <= j
                        continue
                    ref = int(getSlice(s, i, j), 2)
                    assert res == ref
                    assert type(res) == intbv
                    mask = (2 ** (i - j)) - 1
                    assert resi == ref ^ mask
                    assert type(resi) == intbv

    def testGetSliceLeftOpen(self):
        self.seqsSetup()
        for s in self.seqs:
            n = int(s, 2)
            bv = intbv(n)
            bvi = intbv(~n)
            for j in range(0, len(s) + 20):
                res = bv[:j]
                resi = bvi[:j]
                ref = int(getSliceLeftOpen(s, j), 2)
                assert res == ref
                assert type(res) == intbv
                assert resi + ref == -1
                assert type(res) == intbv

    def testSetItem(self):
        self.seqsSetup()
        for s in self.seqs:
            n = int(s, 2)
            for it in (int, intbv):
                for i in range(len(s) + 20):
                    # print i
                    bv0 = intbv(n)
                    bv1 = intbv(n)
                    bv0i = intbv(~n)
                    bv1i = intbv(~n)
                    bv0[i] = it(0)
                    bv1[i] = it(1)
                    bv0i[i] = it(0)
                    bv1i[i] = it(1)
                    ref0 = int(setItem(s, i, '0'), 2)
                    ref1 = int(setItem(s, i, '1'), 2)
                    ref0i = ~int(setItem(s, i, '1'), 2)
                    ref1i = ~int(setItem(s, i, '0'), 2)
                    assert bv0 == ref0
                    assert bv1 == ref1
                    assert bv0i == ref0i
                    assert bv1i == ref1i

    def testSetSlice(self):
        self.seqsSetup()
        toggle = 0
        for s in self.seqs:
            n = int(s, 2)
            for i in range(1, len(s) + 5):
                for j in range(0, len(s) + 5):
                    for v in self.seqv:
                        ext = '0' * (i - j - len(v))
                        extv = ext + v
                        bv = intbv(n)
                        bvi = intbv(~n)
                        val = int(v, 2)
                        toggle ^= 1
                        if toggle:
                            val = intbv(val)
                        try:
                            bv[i:j] = val
                        except ValueError:
                            assert i <= j or val >= 2 ** (i - j)
                            continue
                        else:
                            ref = int(setSlice(s, i, j, extv), 2)
                            assert bv == ref
                        mask = (2 ** (i - j)) - 1
                        vali = val ^ mask
                        try:
                            bvi[i:j] = vali
                        except ValueError:
                            assert vali >= 2 ** (i - j)
                            continue
                        else:
                            refi = ~int(setSlice(s, i, j, extv), 2)
                            assert bvi == refi

    def testSetSliceLeftOpen(self):
        self.seqsSetup()
        toggle = 0
        for s in self.seqs:
            n = int(s, 2)
            for j in range(0, len(s) + 5):
                for v in self.seqv:
                    bv = intbv(n)
                    bvi = intbv(~n)
                    val = int(v, 2)
                    toggle ^= 1
                    if toggle:
                        val = intbv(val)
                    bv[:j] = val
                    bvi[:j] = -1 - val
                    ref = int(setSliceLeftOpen(s, j, v), 2)
                    assert bv == ref
                    refi = ~int(setSliceLeftOpen(s, j, v), 2)
                    assert bvi == refi


class TestIntBvAsInt:

    def seqSetup(self, imin, imax, jmin=0, jmax=None):
        seqi = [imin, imin, 12, 34]
        seqj = [jmin, 12  , jmin, 34]
        if not imax and not jmax:
            l = 2222222222222222222222222222
            seqi.append(l)
            seqj.append(l)
        # first some smaller ints
        for __ in range(100):
            ifirstmax = jfirstmax = 100000
            if imax:
                ifirstmax = min(imax, ifirstmax)
            if jmax:
                jfirstmax = min(jmax, jfirstmax)
            i = randrange(imin, ifirstmax)
            j = randrange(jmin, jfirstmax)
            seqi.append(i)
            seqj.append(j)
        # then some potentially ints
        for __ in range(100):
            if not imax:
                i = randrange(maxint) + randrange(maxint)
            else:
                i = randrange(imin, imax)
            if not jmax:
                j = randrange(maxint) + randrange(maxint)
            else:
                j = randrange(jmin, jmax)
            seqi.append(i)
            seqj.append(j)
        self.seqi = seqi
        self.seqj = seqj

    def binaryCheck(self, op, imin=0, imax=None, jmin=0, jmax=None):
        self.seqSetup(imin=imin, imax=imax, jmin=jmin, jmax=jmax)
        for i, j in zip(self.seqi, self.seqj):
            bi = intbv(int(i))
            bj = intbv(j)
            ref = op(int(i), j)
            r1 = op(bi, j)
            r2 = op(int(i), bj)
            r3 = op(bi, bj)
            # self.assertEqual(type(r1), intbv)
            # self.assertEqual(type(r2), intbv)
            # self.assertEqual(type(r3), intbv)
            assert r1 == ref
            assert r2 == ref
            assert r3 == ref

    def augmentedAssignCheck(self, op, imin=0, imax=None, jmin=0, jmax=None):
        self.seqSetup(imin=imin, imax=imax, jmin=jmin, jmax=jmax)
        for i, j in zip(self.seqi, self.seqj):
            bj = intbv(j)
            ref = int(i)
            ref = op(ref, j)
            r1 = bi1 = intbv(int(i))
            r1 = op(r1, j)
            r2 = int(i)
            r2 = op(r2, bj)
            r3 = bi3 = intbv(int(i))
            r3 = op(r3, bj)
            assert type(r1) == intbv
            assert type(r3) == intbv
            assert r1 == ref
            assert r2 == ref
            assert r3 == ref
            assert r1 is bi1
            assert r3 is bi3

    def unaryCheck(self, op, imin=0, imax=None):
        self.seqSetup(imin=imin, imax=imax)
        for i in self.seqi:
            bi = intbv(i)
            ref = op(i)
            r1 = op(bi)
            # self.assertEqual(type(r1), intbv)
            assert r1 == ref

    def conversionCheck(self, op, imin=0, imax=None):
        self.seqSetup(imin=imin, imax=imax)
        for i in self.seqi:
            bi = intbv(i)
            ref = op(i)
            r1 = op(bi)
            assert type(r1) == type(ref)
            assert r1 == ref

    def comparisonCheck(self, op, imin=0, imax=None, jmin=0, jmax=None):
        self.seqSetup(imin=imin, imax=imax, jmin=jmin, jmax=jmax)
        for i, j in zip(self.seqi, self.seqj):
            bi = intbv(i)
            bj = intbv(j)
            ref = op(i, j)
            r1 = op(bi, j)
            r2 = op(i, bj)
            r3 = op(bi, bj)
            assert r1 == ref
            assert r2 == ref
            assert r3 == ref

    def testAdd(self):
        self.binaryCheck(operator.add)

    def testSub(self):
        self.binaryCheck(operator.sub)

    def testMul(self):
        self.binaryCheck(operator.mul, imax=maxint)  # XXX doesn't work for long i???

    def testTrueDiv(self):
        self.binaryCheck(operator.truediv, jmin=1)

    def testFloorDiv(self):
        self.binaryCheck(operator.floordiv, jmin=1)

    def testMod(self):
        self.binaryCheck(operator.mod, jmin=1)

    def testPow(self):
        self.binaryCheck(pow, jmax=64)

    def testLShift(self):
        self.binaryCheck(operator.lshift, jmax=256)

    def testRShift(self):
        self.binaryCheck(operator.rshift, jmax=256)

    def testAnd(self):
        self.binaryCheck(operator.and_)

    def testOr(self):
        self.binaryCheck(operator.or_)

    def testXor(self):
        self.binaryCheck(operator.xor)

    def testIAdd(self):
        self.augmentedAssignCheck(operator.iadd)

    def testISub(self):
        self.augmentedAssignCheck(operator.isub)

    def testIMul(self):
        self.augmentedAssignCheck(operator.imul, imax=maxint)  # XXX doesn't work for long i???

    def testIFloorDiv(self):
        self.augmentedAssignCheck(operator.ifloordiv, jmin=1)

    def testIMod(self):
        self.augmentedAssignCheck(operator.imod, jmin=1)

    def testIPow(self):
        self.augmentedAssignCheck(operator.ipow, jmax=64)

    def testIAnd(self):
        self.augmentedAssignCheck(operator.iand)

    def testIOr(self):
        self.augmentedAssignCheck(operator.ior)

    def testIXor(self):
        self.augmentedAssignCheck(operator.ixor)

    def testILShift(self):
        self.augmentedAssignCheck(operator.ilshift, jmax=256)

    def testIRShift(self):
        self.augmentedAssignCheck(operator.irshift, jmax=256)

    def testNeg(self):
        self.unaryCheck(operator.neg)

    def testPos(self):
        self.unaryCheck(operator.pos)

    def testAbs(self):
        self.unaryCheck(operator.abs)

    def testInvert(self):
        self.unaryCheck(operator.inv)

    def testInt(self):
        self.conversionCheck(int, imax=maxint)

    def testLong(self):
        self.conversionCheck(int)

    def testFloat(self):
        self.conversionCheck(float)

    # XXX __complex__ seems redundant ??? (complex() works as such?)

    def testOct(self):
        self.conversionCheck(oct)

    def testHex(self):
        self.conversionCheck(hex)

    def testLt(self):
        self.comparisonCheck(operator.lt)

    def testLe(self):
        self.comparisonCheck(operator.le)

    def testGt(self):
        self.comparisonCheck(operator.gt)

    def testGe(self):
        self.comparisonCheck(operator.ge)

    def testEq(self):
        self.comparisonCheck(operator.eq)

    def testNe(self):
        self.comparisonCheck(operator.ne)


class TestIntbvBounds:

    def testConstructor(self):
        assert intbv(40, max=54) == 40
        with pytest.raises(ValueError):
            intbv(40, max=27)

        assert intbv(25, min=16) == 25
        with pytest.raises(ValueError):
            intbv(25, min=27)

    def testSliceAssign(self):
        a = intbv(min=-24, max=34)
        for i in (-24, -2, 13, 33):
            for k in (6, 9, 10):
                a[:] = 0
                a[k:] = i
                assert a == i
        for i in (-25, -128, 34, 35, 229):
            for k in (0, 9, 10):
                with pytest.raises(ValueError):
                    a[k:] = i

        a = intbv(5)[8:]
        for v in (0, 2 ** 8 - 1, 100):
            a[:] = v
        for v in (-1, 2 ** 8, -10, 1000):
            with pytest.raises(ValueError):
                a[:] = v

    def checkBounds(self, i, j, op):
        a = intbv(i)
        assert a == i  # just to be sure
        try:
            exec("a %s int(j)" % op)
        except (ZeroDivisionError, ValueError):
            return  # prune
        if not isinstance(a._val, int):
            return  # prune
        if abs(a) > maxint * maxint:
            return  # keep it reasonable
        if a > i:
            b = intbv(i, min=i, max=a + 1)
            for m in (i + 1, a):
                b = intbv(i, min=i, max=m)
                with pytest.raises(ValueError):
                    exec("b %s int(j)" % op)
        elif a < i:
            b = intbv(i, min=a, max=i + 1)
            exec("b %s int(j)" % op)  # should be ok
            for m in (a + 1, i):
                b = intbv(i, min=m, max=i + 1)
                with pytest.raises(ValueError):
                    exec("b %s int(j)" % op)
        else:  # a == i
            b = intbv(i, min=i, max=i + 1)
            exec("b %s int(j)" % op)  # should be ok

    def checkOp(self, op):
        for i in (0, 1, -1, 2, -2, 16, -24, 129, -234, 1025, -15660):
            for j in (0, 1, -1, 2, -2, 9, -15, 123, -312, 2340, -23144):
                self.checkBounds(i, j, op)

    def testIAdd(self):
        self.checkOp("+=")

    def testISub(self):
        self.checkOp("-=")

    def testIMul(self):
        self.checkOp("*=")

    def testIFloorDiv(self):
        self.checkOp("//=")

    def testIMod(self):
        self.checkOp("%=")

    def testIPow(self):
        self.checkOp("**=")

    def testIAnd(self):
        self.checkOp("&=")

    def testIOr(self):
        self.checkOp("|=")

    def testIXor(self):
        self.checkOp("^=")

    def testILShift(self):
        self.checkOp("<<=")

    def testIRShift(self):
        self.checkOp(">>=")


class TestIntbvCopy:

    def testCopy(self):

        for n in (intbv(), intbv(34), intbv(-12, min=-15), intbv(45, max=65),
                  intbv(23, min=2, max=47), intbv(35)[3:]):
            a = intbv(n)
            b = copy(n)
            c = deepcopy(n)
            for m in (a, b, c):
                assert n == m
                assert n._val == m._val
                assert n.min == m.min
                assert n.max == m.max
                assert len(n) == len(m)



from myhdl import (block, Signal, always, modbv, delay, instance, StopSimulation, instances, Simulation)

WIDTH = 4

clk = Signal(bool(0))
x = Signal(modbv(0)[WIDTH:])
y = Signal(modbv(0)[WIDTH:])
z = Signal(modbv(0)[WIDTH:])


@always(delay(5))
def tb_clk_gen():
    clk.next = not clk


@always(clk.posedge)
def inc():
    y.next = x + 1


@always(clk.posedge)
def dec():
    z.next = x - 1


@instance
def tb_stimulus():
    # My logic happens on posedge, so I'll perform all checks on negedge.
    yield clk.negedge
    for x_val in range(-2 ** WIDTH, 2 ** WIDTH):
        # print('x_val={} x.next={}'.format(x_val, x_val % 2**WIDTH))
        x.next = x_val % 2 ** WIDTH
        yield clk.negedge
        assert y == (x_val + 1) % 2 ** WIDTH, 'y={} but expected {}'.format(y, (x_val + 1) % 2 ** WIDTH)
        assert z == (x_val - 1) % 2 ** WIDTH, 'z={} but expected {}'.format(z, (x_val - 1) % 2 ** WIDTH)
    print('OK!')
    raise StopSimulation


tb = instances()


def test_bug_44():
    print(instances())
    Simulation(tb).run()

